/* Electronic Voting Project
 * Mark A. Herschberg (hershey@mit.edu)
 */

package evote.crypto;

import java.math.BigInteger;
import java.security.*;
import java.util.Random;

import evote.util.constants.*;
import evote.util.genrand.*;


/**
  This class provides encryption, decryption, signing, and confirmation of
  signatures based on the RSA cryptographic methods.  It requires an RSAKey.
  First an RsaEncryption object must be instaniated (usually with a key),
  before any of these operations can be done.
  */
public class RsaEncryption implements EncryptClass, Signer {
  RsaKeys keys;
  private NoKeyException nke;
  
  /**
    This creates a blank RsaEncryption object, with has a blank RSA key.
    */
  public RsaEncryption() {
    keys = new RsaKeys();
  }
  
  /**
    This creates an RsaEncryption object using the byte array as a seed to
    generate the RSA key.
    */
  public RsaEncryption(byte[] seed) {
    SecureRandom SRNG = new SecureRandom(seed);
    keys = new RsaKeys(seed);
  }
  
  /**
    This creates an RsaEncryption object using the GenRand object to generate
    the RSA key.
    */
  public RsaEncryption(GenRand RNG) {
    keys = new RsaKeys(RNG);
  }
  
  /**
    This creates an RsaEncryption object using the specified RSA key.
    */
  public RsaEncryption(RsaKeys givenKeys) {
    keys = givenKeys;
  }
  
  /**
    This creates an RsaEncryption object with an RSA Key with the given e, d,
    and n.  (p and q are not set.)
    */
  public RsaEncryption(BigInteger e, BigInteger d, BigInteger n) {
    keys = new RsaKeys();
    keys.SetRsaP(null);
    keys.SetRsaQ(null);
    keys.SetRsaE(e);
    keys.SetRsaD(d);
    keys.SetRsaN(n);
  }
  
  /**
    This creates an RsaEncryption object with an RSA Key with the given e, and 
    n.  (p, q, and d are not set.)
    */
  public RsaEncryption(BigInteger e, BigInteger n) {
    keys = new RsaKeys();
    keys.SetRsaP(null);
    keys.SetRsaQ(null);
    keys.SetRsaE(e);
    keys.SetRsaD(null);
    keys.SetRsaN(n);
  }
  
  public EncryptionKey getEncryptionKey() throws Exception {
    if ( (keys.getRsaE() == null) || (keys.getRsaN() == null) ) {
      NoKeyException nke = new NoKeyException("Encrypt: e=" + (keys.getRsaE() == null ? "NULL" : (keys.getRsaE()).toString()) + " n=" + (keys.getRsaN() == null ? "NULL" : (keys.getRsaN()).toString()));
      throw(nke);
    }
    else {
      return (new PublicKey(keys.getRsaE(), keys.getRsaN()));
    }
  }
  
  /**
    Returns a PublicKey.  Throws a NoKeyException if e or n is null.
    */
  public EncryptionKey getEncryptKey() throws Exception {
    if ( (keys.getRsaE() == null) || (keys.getRsaN() == null) ) {
      NoKeyException nke = new NoKeyException("Encrypt: e=" + (keys.getRsaE() == null ? "NULL" : (keys.getRsaE()).toString()) + " n=" + (keys.getRsaN() == null ? "NULL" : (keys.getRsaN()).toString()));
      throw(nke);
    }
    else {
      return (new PublicKey(keys.getRsaE(), keys.getRsaN()));
    }
  }
  
  /**
    Returns a PrivateKey.  Throws a NoKeyException if d or n is null.
    */
  public EncryptionKey getDecryptKey() throws Exception {
    if ( (keys.getRsaD() == null) || (keys.getRsaN() == null) ) {
      NoKeyException nke = new NoKeyException("Encrypt: d=" + (keys.getRsaD() == null ? "NULL" : (keys.getRsaD()).toString()) + " n=" + (keys.getRsaN() == null ? "NULL" : (keys.getRsaN()).toString()));
      throw(nke);
    }
    else {
      return (new PrivateKey(keys.getRsaD(), keys.getRsaN()));
    }
  }
  
  /**
    Returns the byte array RSA encrypted with the RSA key instantiated with the
    encryption object.  Throws a NoKeyException if e or n is null.
    */
  public byte[] encrypt(byte[] input) {
    BigInteger plain, enc;
    plain = new BigInteger(input);
    try {
      enc = plain.modPow(keys.getRsaE(), keys.getRsaN());
    }
    catch (ArithmeticException exp) {
      System.out.println("You've got a problem with RsaEncrypt(byte[] input)");
      enc = new BigInteger("-1");
    }
    return enc.toByteArray();
  }
  
  /**
    Returns the byte array RSA dencrypted with the RSA key instantiated with
    the encryption object.  Throws a NoKeyException if d or n is null.
    */
  public byte[] decrypt(byte[] input) {
    BigInteger plain, enc;
    enc = new BigInteger(input);
    try {
      plain = enc.modPow(keys.getRsaD(), keys.getRsaN());
    }
    catch (ArithmeticException exp) {
      System.out.println("You've got a problem with RsaDecrypt(byte[] input)");
      plain = new BigInteger("-1");
    }
    return plain.toByteArray();
  }
  
  /**
    Another name for decrypt.
    */
  public byte[] sign(byte[] b) {
    return this.decrypt(b);
  }
  
  /**
    Another name for encrypt.
    */
  public byte[] unsign(byte[] b) throws Exception {
    return this.encrypt(b);
  }
  
  /**
    Checks the signature of signed byte array by Unsigning it and comparing it
    to the plaintext.  (Assumes the signature was created with the RSA key
    instantiated with the RsaEncrption object.
    */
  public boolean verify(byte[] data, byte[] supposed_sign) {
    boolean good = false;
    try {
      if ((new BigInteger(data)).compareTo(new BigInteger(unsign(supposed_sign))) == 0) {
	good = true;
      } else {
	good = false;
      }
    } catch (Exception exp)   {
      System.out.println("Error verifying signature");
    }
    return good;
  }
  
}
