/* Electronic Voting Project
 * Mark A. Herschberg (hershey@mit.edu)
 */

package evote.crypto;

import java.math.BigInteger;
import java.security.*;
import java.util.Random;

import evote.util.constants.*;
import evote.util.genrand.*;


/**
  The committment scheme used is based on HMAC-SHA which involves a double
  hash with two keys. Specifically
          committment = SHA(key2, ( SHA(key1, input) ) )
	  */
public class ShaCommitment implements Commitment {
  ShaCommitmentKeys keys;
  Constants cons = new Constants();
  
  /**
    Creates an blank ShaCommitment object.
    */
  public ShaCommitment() {}
  
  /**
    Creates a ShaCommitment object using keys generated from the byte array
    seed using SecureRandom.
    */
  public ShaCommitment(byte[] seed) {
    this(new SecureRandom(seed));
  }
  
  public ShaCommitment(SecureRandom srand) {
    keys.k1 = new BigInteger(cons.hashKeyLength, srand);
    keys.k2 = new BigInteger(cons.hashKeyLength, srand);
  }

  /**
    Creates a ShaCommitment object using keys generated using the GenRand
    object.
    */
  public ShaCommitment(GenRand RNG) {
    byte[] tempbytes = RNG.nextBytes(cons.hashKeyLength);
    SecureRandom SRNG = new SecureRandom(RNG.nextBytes(100));
    SRNG.nextBytes(tempbytes);
    BigInteger tk1 = new BigInteger(1, tempbytes);
    SRNG.nextBytes(tempbytes);
    BigInteger tk2 = new BigInteger(1, tempbytes);
    keys = new ShaCommitmentKeys(tk1, tk2);
  }
  
  /**
    Creates a ShaCommitment object using keys passed to it.
    */
  public ShaCommitment(ShaCommitmentKeys givenKeys) {
    keys = givenKeys;
  }
  
  /**
    This method performs a single hash on the byte array using the single key
    given by prepending the key to the input.
    */
  public byte[] hashWithKey(byte[] input, byte[] key) {
    int i;
    byte[] output = {5,6};
    int meslength = input.length;
    int keylength = key.length;
    byte[] hashMe = new byte[meslength + keylength];
    for (i = 0; i < keylength; i++) {
      hashMe[i] = key[i];
    }
    for (i = 0; i < meslength; i++) {
      hashMe[keylength + i] = input[i];
    }
    try {
      // this here needs to be REPLACED with new SHA
      // output = (MessageDigest.getInstance("SHA")).digest(hashMe);
      SHADigest sd= new SHADigest();
      output= sd.performHash(hashMe);
    }
    catch (Exception e) {}
    return output;
  }
  
  /**
    This performs the HMAC-SHA by using two calls to hashWithKey.
    */
  public byte[] commit(byte[] input) {
    byte[] intermediate = this.hashWithKey(input, (keys.getKey1()).toByteArray());
    return (new BigInteger(1,this.hashWithKey(intermediate, (keys.getKey2()).toByteArray()))).toByteArray(); // force it to be positive
  }
  
  public byte[] uncommit(byte[] b) throws Exception {
    throw (new Exception("I don't go both ways!"));  //because we can't uncommit SHA
  }
  
  /**
    Returns the keys used by this instaniation of ShaCommitment.
    */
  public CommitmentKeys getKeys() {
    return keys;
  }
}
