/*
 *  Election.java
 *  Ben Adida (ben@mit.edu)
 *  26 Oct 97
 */

package evote.election;

import java.io.*;
import evote.identity.*;
import evote.question.*;

import evote.anonymizer.*;
import evote.counter.*;
import evote.admin.*;
import evote.voter.*;
import evote.commissioner.*;


/**
 *  The Election object is analogous to a paper ballot.  It contians
 *  the questions and choices of the election.
 */

/*
 * The data structure here is a 2-dimensional array of booleans
 * which define the matching between the question lists and user lists
 * The first dimension is the user list
 * The second dimension is the question list
 */

public class Election implements Serializable {

  /* Storage of Questions and Stuff */
  private QuestionList[] questionLists;
  private int questionListSize;
  private UserList[] userLists;
  private int userListSize;

  private boolean[][] matching;
  
  /* Parameters for all Servers */
  private VoterParameters voterParameters;
  private CounterParameters counterParameters;
  private AdminParameters adminParameters;
  private AnonymizerParameters anonymizerParameters;
  private CommissionerParameters commissionerParameters;

  public Election() {
    questionLists= new QuestionList[4];
    userLists= new UserList[4];
    matching= new boolean[4][4];
      
    questionListSize= 0;
    userListSize= 0;

    // set up empty parameters
    voterParameters= new VoterParameters();
    counterParameters= new CounterParameters();
    adminParameters= new AdminParameters();
    anonymizerParameters= new AnonymizerParameters();
    commissionerParameters= new CommissionerParameters();
  }
  
  public void setVoterParameters(VoterParameters vp) {
    voterParameters= vp;
  }

  public void setCounterParameters(CounterParameters cp) {
    counterParameters= cp;
  }

  public void setAdminParameters(AdminParameters ap) {
    adminParameters= ap;
  }

  public void setAnonymizerParameters(AnonymizerParameters ap) {
    anonymizerParameters= ap;
  }

  public void setCommissionerParameters(CommissionerParameters cp) {
    commissionerParameters= cp;
  }

  public VoterParameters getVoterParameters() {
    return(voterParameters);
  }

  public CounterParameters getCounterParameters() {
    return(counterParameters);
  }

  public AdminParameters getAdminParameters() {
    return(adminParameters);
  }

  public AnonymizerParameters getAnonymizerParameters() {
    return(anonymizerParameters);
  }

  public CommissionerParameters getCommissionerParameters() {
    return(commissionerParameters);
  }

  public void clearUsersAndQuestions() {
    questionLists= new QuestionList[4];
    userLists= new UserList[4];
    matching= new boolean[4][4];
      
    questionListSize= 0;
    userListSize= 0;
  }

  public UserList[] getUserLists() {
    UserList[] returnList= new UserList[userListSize];
    System.arraycopy(userLists, 0, returnList, 0, userListSize);
    return(returnList);
  }

  public QuestionList[] getQuestionLists() {
    QuestionList[] returnList= new QuestionList[questionListSize];
    System.arraycopy(questionLists, 0, returnList, 0, questionListSize);
    return(returnList);
  }

  private void growQuestionList() {
    QuestionList[] newQuestionLists= new QuestionList[questionLists.length *2];
    System.arraycopy(questionLists, 0, newQuestionLists, 0, questionLists.length);
    
    for (int i=0; i<userLists.length; i++) {
      boolean[] newArray= new boolean[newQuestionLists.length];
      System.arraycopy(matching[i], 0, newArray,0, questionLists.length);
      matching[i]= newArray;
    }

    questionLists= newQuestionLists;
  }

  private void growUserList() {
    UserList[] newUserLists= new UserList[userLists.length *2];
    System.arraycopy(userLists,0,newUserLists,0,userLists.length);
    
    boolean[][] newMatching= new boolean[newUserLists.length][];
    System.arraycopy(matching, 0, newMatching, 0, userLists.length);
    for (int i=userLists.length; i<newUserLists.length; i++)
      newMatching[i]= new boolean[questionLists.length];
  
    matching= newMatching;
    userLists= newUserLists;
  }

  private int findUserList(UserList ul) {
    for (int i=0; i<userListSize; i++)
      if (userLists[i] == ul)
	return(i);

    return(-1);
  }

  private int findQuestionList(QuestionList ql) {
    for (int i=0; i<questionListSize; i++)
      if (questionLists[i] == ql)
	return(i);

    return(-1);
  }
  
  public synchronized void addUserList(UserList ul) {
    if (userListSize >= userLists.length)
      growUserList();

    
    userLists[userListSize]= ul;
    userListSize+= 1;
  }

  public synchronized void delUserList(int pos) {
    if (pos > userListSize)
      return;
    
    /* Adjust size */
    int newLength= userLists.length;
    if (userLists.length> (4*(userListSize-1)))
      newLength= userLists.length/2;

    UserList[] newUserLists= new UserList[newLength];
    
    /* copy things over */
    for (int i=0; i<pos; i++)
      newUserLists[i]= userLists[i];
    for (int i=pos+1; i<userListSize; i++)
      newUserLists[i-1]= userLists[i];

    /* Fix up the matching table */
    /* Since userlist is first dimension, need only do the same thing
     * as for the userlist array itself
     */
    boolean[][] newMatching= new boolean[newLength][questionLists.length];
    for (int i=0; i<pos; i++)
      newMatching[i]= matching[i];
    for (int i= pos+1; i<userListSize; i++)
      newMatching[i-1]= matching[i];
    
    userListSize-= 1;
    userLists= newUserLists;
    matching= newMatching;
  }

  public synchronized void addQuestionList(QuestionList ql) {
    if (questionListSize == questionLists.length)
      growQuestionList();
    
    questionLists[questionListSize]= ql;

    /* let question list know which one it is */
    questionLists[questionListSize].setID(questionListSize);
    questionLists[questionListSize].renumberQuestions();

    questionListSize+= 1;
  }

  public synchronized void delQuestionList(int pos) {
    if (pos > questionListSize)
      return;

    /* Adjust Size of question list array */
    int newLength= questionLists.length;
    if (questionLists.length > (4*(questionListSize-1)))
      newLength= questionLists.length/2;

    QuestionList[] newQuestionLists= new QuestionList[newLength];

    /* copy over */
    for (int i=0; i<pos; i++)
      newQuestionLists[i]= questionLists[i];
    for (int i=pos+1; i<questionListSize; i++)
      newQuestionLists[i-1]= questionLists[i];

    /* Adjust the matching array */
    /* This is tricky because question lists are second dimension */

    boolean[][] newMatching= new boolean[userLists.length][newLength];
    for (int i=0; i<userListSize; i++) {
      for (int j=0; j<pos; j++)
	newMatching[i][j]= matching[i][j];
      for (int j=pos+1; j<questionListSize; j++)
	newMatching[i][j-1]= matching[i][j];
    }
    
    questionListSize-= 1;
    questionLists= newQuestionLists;
    matching= newMatching;
  }

  public synchronized void setMatching(int i, int j, boolean match) {
    matching[i][j]= match;
  }

  public synchronized void setMatching(UserList ul, QuestionList ql, boolean match) {
    int i= findUserList(ul);
    
    if (i<0)
      return;

    int j= findQuestionList(ql);

    if (j<0)
      return;

    setMatching(i,j,match);
  }

  public synchronized boolean getMatching(int i, int j) {
    return(matching[i][j]);
  }
  
  public synchronized boolean getMatching(UserList ul, QuestionList ql) {
    
    int i= findUserList(ul);
    
    if (i<0)
      return(false);

    int j= findQuestionList(ql);

    if (j<0)
      return(false);
    
    return(getMatching(i,j));
  }

  public synchronized UserList getAllUsers() {
    if (this.userListSize == 0)
      return(null);

    UserList return_ul= (UserList)userLists[0].clone();
    
    boolean problem= false;

    for (int i=0; i<this.userListSize; i++) {
      try {
	return_ul.append(userLists[i]);
      } catch(UserNameTakenException unte) {
	problem= true;
      }
    }

    return(return_ul);
  }

  public QuestionList getQuestionList(int pos) {
    return(questionLists[pos]);
  }

  public UserList getUserList(int pos) {
    return(userLists[pos]);
  }

  public synchronized QuestionList[] getQuestionLists(UserList ul) {

    // find the userlist
    int pos= findUserList(ul);
    
    if (pos<0)
      return(null);

    int length=0;
    // one pass for length
    for (int i=0; i<questionListSize; i++)
      if (matching[pos][i])
	length+= 1;

    QuestionList[] returnList= new QuestionList[length];
    int return_pos= 0;

    // second pass for construction
    for (int i=0; i<questionListSize; i++)
      if (matching[pos][i])
	returnList[return_pos++]= questionLists[i];

    return(returnList);
  }

  public synchronized UserList[] getUserLists(QuestionList ql) {
    
    // find the questionlist
    int pos= findQuestionList(ql);

    if (pos<0)
      return(null);

    int length=0;
    // one pass for length
    for (int i=0; i<userListSize; i++)
      if (matching[pos][i])
	length+= 1;

    UserList[] returnList= new UserList[length];
    int return_pos= 0;

    // second pass for construction
    for (int i=0; i<userListSize; i++)
      if (matching[pos][i])
	returnList[return_pos++]= userLists[i];
    
    return(returnList);
  }    

  public synchronized QuestionList makeUserQuestionList(User u) {
    QuestionList returnList= null;

    boolean[] alreadyIncluded= new boolean[questionListSize];
    for (int i=0; i<questionListSize; i++)
      alreadyIncluded[i]= false;

    for (int i=0; i<userListSize; i++) {
      if (userLists[i].hasUser(u)) {
	System.out.println("ADMIN: " + u.getUserName() + " is in " + userLists[i].getName());
	QuestionList[] lists= getQuestionLists(userLists[i]);
	System.out.println("That particular userlist has " + lists.length + " questionlist(s)");

	for (int j=0; j<lists.length; j++) {
	  if (returnList == null)
	    returnList= (QuestionList) lists[j].clone();
	  else {
	    if (alreadyIncluded[findQuestionList(lists[j])])
	      continue;
	    returnList.append(lists[j]);
	  }
	  alreadyIncluded[findQuestionList(lists[j])]= true;
	}
      }
    }

    return(returnList);
  }
  
  // a method for getting just the IDs of the questionlists
  // for a particular user
  public int[] getQuestionListNumbers(User u) {
    boolean[] which_qls= new boolean[questionListSize];
    
    for (int i=0; i<userListSize; i++) {
      if (userLists[i].hasUser(u)) {
	for (int j=0; j<questionListSize; j++) {
	  which_qls[j] |= matching[i][j];
	}
      }
    }

    int count=0;
    for (int i=0; i<questionListSize; i++) {
      if (which_qls[i])
	count++;
    }

    int[] returnList= new int[count];

    count=0;
    for (int i=0; i<questionListSize; i++) {
      if (which_qls[i]) {
	returnList[count]= i;
	count++;
      }
    }

    return(returnList);
  }

  /**
   *  Serializes the Election object.
   */
  public byte[] serialize() throws Exception {
    byte[] ser_election;
    ByteArrayOutputStream baos= new ByteArrayOutputStream();
    try {
      ObjectOutputStream oos = new ObjectOutputStream(baos);
      oos.writeObject(this);
      ser_election = baos.toByteArray();
      return(ser_election);
    }
    catch(IOException ie) {
      throw(new Exception("couldn't serialize"));
    }
  }
  
  /**
   *  De-serializes the Election object.
   */
  public static Election deserialize(byte[] b) throws Exception {
    try {
      ByteArrayInputStream bais= new ByteArrayInputStream(b);
      ObjectInputStream ois= new ObjectInputStream(bais);
      Election e = (Election)ois.readObject();
      return(e);
    }
    catch(Exception e) {
      throw(new Exception("Format Error"));
    }
  }  

  /**
   * Clone the election object
   */
  public Object clone() {
    Election otherSelf= new Election();
    
    otherSelf.questionLists= (QuestionList[]) this.questionLists;
    otherSelf.questionListSize= this.questionListSize;
    otherSelf.userLists= (UserList[]) this.userLists;
    otherSelf.userListSize= this.userListSize;
    otherSelf.matching= (boolean[][]) this.matching.clone();

    otherSelf.voterParameters= (VoterParameters) this.voterParameters.clone();
    otherSelf.counterParameters= (CounterParameters) this.counterParameters.clone();
    otherSelf.adminParameters= (AdminParameters) this.adminParameters.clone();
    otherSelf.anonymizerParameters= (AnonymizerParameters) this.anonymizerParameters.clone();
    otherSelf.commissionerParameters= (CommissionerParameters) this.commissionerParameters.clone();

    return(otherSelf);
  }

  /**
   * Make hashable
   */
  public void makeHashable() {
    userLists= null;
    matching= null;
    userListSize= 0;
    
    voterParameters.clearPrivate();
    counterParameters.clearPrivate();
    adminParameters.clearPrivate();
    anonymizerParameters.clearPrivate();
  }
}

