/* Electronic Voting Project
 * Server Code
 * written by Ben Adida (ben@mit.edu)
 */

package evote.network.server;
import evote.demo.*;

import java.io.*;
import java.net.*;
import evote.util.queue.*;
import evote.util.constants.*;
import evote.lang.*;
import java.util.*;

/**
 * The Server Layer
 *
 * This is the network server abstraction. Given a port number and a 
 * connection handler, the server takes care of thread creationg and 
 * request queuing, calling on the connection handler when a connection
 * needs to be dealt with.
 * <p>
 * A Server has a fixed number of threads to handle connections, and a queue
 * of requests when all threads are busy. Threads are recycled to avoid the
 * overhead of creating and destroying threads. 
 * <p>
 * The Server also contains a Verifier, a module that keeps a list of all the 
 * clients that have connected to the server, and refuses connections of
 * clients who have connected too often, thereby avoiding a large class
 * of denial of service attacks
 * <p>
 * In terms of GUI, the Server can have ConnectionListeners, and 
 * BlackListListeners. Any new connection, or lost connection, will result
 * in a notification of all the ConnectionListeners, while any change to the
 * BlackList will result in a notification of the BlackListListeners.
 *
 * @see evote.network.server.ConnectionHandler
 * @see evote.network.server.ConnectionListener
 * @see evote.network.server.BlackListListener
 */

public class Server implements Runnable, TimeControllable {
    protected Thread my_thread;
    protected ServerSocket my_socket;
    protected RequestScheduler my_scheduler;
    protected RequestValidator my_validator;
    protected ConnectionHandler conn_handler;
    protected ServerInterface my_interface;

    private Vector blacklist_listeners;
    private Vector connection_listeners;
    private String my_name;
    private Log my_log;
    private int my_timeout= 15000;
  
    protected boolean shutdown_in_progress;
    private int my_port;

    private Date startDate, endDate;
    private TimeController myTimeController;

    // for demo purposes
    private DemoInterface myDemoInterface;
  
    /**
     * This constructs a new Server
     *
     * @argument name The name of the server that will appear on the GUI and 
     * that will be used as the log filename.
     * @argument port The port number
     * @argument ch The connection handler
     * @argument num_threads The number of threads used in this server.
     * @argument queue_size The size of the queue of requests that are 
     * not handled
     * @return The new Server
     * @exception evote.network.server.ServerException 
     *       If anything goes wrong (can't bind port, can't create threads...),
     *       this exception is thrown.
     */
    public Server(String name, int port, ConnectionHandler ch, int num_threads, int queue_size, boolean interface_p) throws ServerException {
    
	my_port= port;

	try {
	    my_socket= new ServerSocket(port);
	}
	catch(Exception e) {
	    throw(new ServerException(e.toString()));
	}
	conn_handler= ch;
	my_scheduler= new RequestScheduler(this, conn_handler, num_threads, queue_size);
	my_validator= new RequestValidator(this,5000000);

	if (interface_p)
	    my_interface= new ServerInterface(name,this);
	else 
	    my_interface= null;
    
	blacklist_listeners= new Vector();
	connection_listeners= new Vector();

	if (my_interface != null) {
	    this.addBlackListListener(my_interface);
	    this.addConnectionListener(my_interface);
	}

	this.my_name= name;
    
	try {
	    my_log= new Log(my_name+"LOGGER.txt");
	}
	catch(LogException le) {
	    throw(new ServerException("Couldn't open log"));
	}

    }

    public Server(String name, int port, ConnectionHandler ch, int num_threads, int queue_size) throws ServerException {
	this(name,port,ch,num_threads,queue_size, true);
    }

    /**
     * This constructs the default server, with 10 threads, and queue size 50.
     *
     * @argument name The name of the server
     * @argument port The port number
     * @argument ch The Connection handler
     * @return the new server.
     * @exception evote.network.server.ServerException
     *     If anything goes wrong, this exception is thrown
     */
    public Server(String name, int port, ConnectionHandler ch) throws ServerException {
	this(name, port, ch, 10, 50);
    }

    public void startDemo() {
	myDemoInterface= new DemoInterface(my_name);
    }

    public void stopDemo() {
	if (myDemoInterface != null)
	    myDemoInterface.destroy();
	myDemoInterface= null;
    }

    public void setStatus(String status) {
	if (myDemoInterface != null)
	    myDemoInterface.setStatus(status);
    }

    public void clearStatus() {
	if (myDemoInterface != null)
	    myDemoInterface.clearStatus();
    }	

    public void incrementCount() {
	if (myDemoInterface != null)
	    myDemoInterface.incrementCount();
    }

    /** 
     * This sets the start and end time
     */
    public void setTimes(Date start, Date end) {
	startDate= start;
	endDate= end;
    }

    /**
     * This orders the server to start
     *
     */
    public void start() {
	// start the TimeController on us.
	if ((startDate != null) && (endDate !=null)) {
	    myTimeController= new TimeController(this, startDate, endDate);
	    if (my_interface == null)
		myTimeController.disableGUI();
	} else {
	    timeStart();
	}
    }

    /**
     * The TimeControllable timeStart()
     */
    public void timeStart() {
	my_thread= new Thread(this);
	my_thread.start();
    }

    /**
     * The TimeControllable timeStop()
     */
    public void timeStop() {
	System.out.println("System stopping -- SERVER LAYER");
	shutdown();
    }

    /**
     * This adds the given entry to the log.
     *
     * @argument le The log entry to add
     */
    public void log(LogEntry le) {
	try {
	    my_log.addLogEntry(le);
	}
	catch(LogException excp) {
	}
    }
  
    /**
     * This method is available for future servers that might
     * have two separate logs
     * 
     * @argument le The log entry to add.
     */
    public void logError(LogEntry le) {
	this.log(le);
    }

    /** 
     * This sets the timeout on future connections that will be created.
     * Currently open connections are not affected by this change
     *
     * @argument t The number of milliseconds for the timeout
     */
    public void setTimeout(int t) {
	my_timeout= t;
    }

    /**
     * This returns the current value of the timeout.
     *
     * @return The timeout
     */
    public int getTimeout() {
	return(my_timeout);
    }
  
    /**
     * This "clears the record" for an InetAddress, effectively removing it 
     * from the Black List. If the address is not in the Black List, this 
     * method is silent, and performs nothing.
     *
     * @argument ia The InetAddress to clear
     */
    public void clearRecord(InetAddress ia) {
	my_validator.clearRecord(ia);
    }

    /** 
     * This "clears the record" for a hostname, designated by a String.
     * If the hostname is not on the Black List, nothing happens.
     *
     * @argument s The string of the hostname whose record is being cleared.
     */
    public void clearRecord(String s) {
	try {
	    this.clearRecord(InetAddress.getByName(s));
	}
	catch(UnknownHostException e) {
	    /* Log */
	    this.log(new LogEntry(LogEntry.WARNING_LOG,"Server Layer: couldn't find host " + s + " to reenable"));
	}
    }
    
    /**
     * This returns the number of busy threads.
     * 
     * @return The number of busy threads.
     */
    public int getNumBusyThreads() {
	return(my_scheduler.getNumBusyThreads());
    }

    /** 
     * This returns the current number of requests in the
     * queue.
     *
     * @return the number of requests queued
     */
    public int getQueueSize() {
	return(my_scheduler.getQueueSize());
    }

    /**
     * This gives a string status for each of the threads
     * If a thread is free, the null string is returned for that 
     * thread. Otherwise, the hostname that the thread is connected
     * to is given.
     * 
     * @return the array of strings each giving the status for one thread.
     */
    public String[] getAllThreadsStatus() {
	return(my_scheduler.threadsStatus());
    }

    /**
     * This returns the actual listing of the black list, by InetAddress
     *
     * @return an array of InetAddress that lists the blacklisted internet
     * addresses
     */
    public InetAddress[] getBlackList() {
	RequestRecordList rrl= my_validator.getBlackList();
	RequestRecordList rrl2= rrl;
	int counter=0;
	while(rrl2.next!=null) {
	    counter+= 1;
	    rrl2= rrl2.next;
	}
	InetAddress[] black_inet= new InetAddress[counter];
	for (int i= 0; i< counter; i++){
	    rrl= rrl.next;
	    black_inet[i]= rrl.my_record.getInetAddress();
	}
	return(black_inet);
    }

    /**
     * This returns the Black List, but in String format
     * 
     * @return an array of strings giving the black list entries
     */
    public String[] getBlackListString(int start) {
	InetAddress[] ia= this.getBlackList();
	if (start>ia.length) start=0;
	String[] ia_string= new String[ia.length-start];
	for (int i=0; i<ia_string.length; i++)
	    ia_string[i]= ia[i+start].getHostName();

	return(ia_string);
    }

    /************************************/
    /*  EVENT handlers for the server   */
    /************************************/

    /**
     * This adds a connection listener.
     *
     * @argument cl The new connection listener
     */
    public void addConnectionListener(ConnectionListener cl) {
	connection_listeners.addElement(cl);
    }

    /**
     * This removes a connection listener.
     *
     * @argument cl The connection listener to remove.
     */
    public void removeConnectionListener(ConnectionListener cl) {
	connection_listeners.removeElement(cl);
    }

    /** 
     * This adds a Black List listener.
     *
     * @argument bll The new black list listener.
     */
    public void addBlackListListener(BlackListListener bll) {
	blacklist_listeners.addElement(bll);
    }

    /**
     * This removes a black list listener
     * 
     * @argument bll The black list listener to remove
     */
    public void removeBlackListListener(BlackListListener bll) {
	blacklist_listeners.removeElement(bll);
    }

    protected void blackList(InetAddress ia) {
	BlackListEvent ble= new BlackListEvent(ia);

	/* Log This Event */
	this.log(new LogEntry(LogEntry.NORMAL_LOG, "Server Layer: blacklisting " + ia.toString()));

	Enumeration enum= blacklist_listeners.elements();
	while (enum.hasMoreElements()) {
	    ((BlackListListener)enum.nextElement()).addBlackListedElement(ble);
	}
    }
   
    protected void blackListRemove(InetAddress ia) {
	BlackListEvent ble= new BlackListEvent(ia);

	/* Log This Event */
	this.log(new LogEntry(LogEntry.NORMAL_LOG, "Server Layer: reenabling " + ia.toString()));

	Enumeration enum= blacklist_listeners.elements();
	while (enum.hasMoreElements()) {
	    ((BlackListListener)enum.nextElement()).removeBlackListedElement(ble);
	}
    }

    protected void connectionStart(int t_num, InetAddress ia) {
	ConnectionEvent ce= new ConnectionEvent(t_num, ia);
	Enumeration enum= connection_listeners.elements();
    
	while(enum.hasMoreElements()) {
	    ((ConnectionListener)enum.nextElement()).connectionStarted(ce);
	}
    }
  
    protected void connectionEnd(int t_num) {
	ConnectionEvent ce= new ConnectionEvent(t_num);
	Enumeration enum= connection_listeners.elements();
    
	while(enum.hasMoreElements()) {
	    ((ConnectionListener)enum.nextElement()).connectionEnded(ce);
	}
    }

    protected void connectionQueued() {
	ConnectionEvent ce= new ConnectionEvent(-1);
	Enumeration enum= connection_listeners.elements();
    
	while(enum.hasMoreElements()) {
	    ((ConnectionListener)enum.nextElement()).connectionQueued(ce);
	}
    }

    protected void connectionDequeued() {
	ConnectionEvent ce= new ConnectionEvent(-1);
	Enumeration enum= connection_listeners.elements();
    
	while(enum.hasMoreElements()) {
	    ((ConnectionListener)enum.nextElement()).connectionDequeued(ce);
	}
    }
  
    /*********************/
    /*********************/
  

    /**
     * This destroys the Server and shuts down the system
     */
    public void destroy() {
    
	/* Clean things up */

	/* Tell the connection handler we're done */
	conn_handler.serverShutdown();

	System.exit(0);
    }

    /** 
     * This requests a shutdown of the server
     * This waits for the end of all connections, and then shuts down
     * While the server is in the progress of shutting down (waiting
     * for all connections to end), shutdown can be aborted.
     */
    public void shutdown() {
	shutdown_in_progress= true;
	System.out.println("Shutdown now in Progress");

	/* Create a Dummy Connection to close things off */
	try {
	    new DummyConnection("localhost", this.my_port);
	}
	catch(Exception e) {
	}
	System.out.println("Shutdown should be happening");
    }

    /**
     * This aborts Shutdown
     */
    public void abortShutdown() {
	shutdown_in_progress= false;
    }

    /**
     * This creates a new Connection, given a socket.
     * This method exists so that the precise connection type
     * is determined by the Server.
     */
    public Connection newConnection(Socket s) throws ConnectionException {
	return(new Connection(s));
    }

    public void run() {

	/* Log beginning of activity */
	this.log(new LogEntry(LogEntry.NORMAL_LOG, "Server Layer: Server Started"));

	Socket incoming_s;
	try {
	    while(true) {

		/* Reconstruct the server socket if needed */
		if (my_socket==null)
		    my_socket= new ServerSocket(my_port);

		try {
		    /* Normal operation of the server */
		    while(!shutdown_in_progress) {
			System.out.println("waiting for socket accept");
			incoming_s= my_socket.accept();
			System.out.println("accepted SOCKET connect");
			if (my_validator.validate(incoming_s)) {
			    System.out.println("validated");
			    my_scheduler.handle(incoming_s);
			} 
			else {
			    incoming_s.close();
			    System.out.println("INVALID");
			}
		    }
		}
		catch(IOException ioe) {
		    System.out.println("Server exception");
		    if (!shutdown_in_progress)
			throw(ioe);	
		    System.out.println("which is normal because server is shutting down");
		}

		/* Server in process of shutting down */
		Thread.sleep(1000);

		/* Make sure you still want to shut down */
		while(shutdown_in_progress) {
		    if (my_scheduler.getNumBusyThreads()==0) {
			System.out.println("Really destroying things now!");
			if (my_interface != null)
			    my_interface.destroy();

			/* Log */
			this.log(new LogEntry(LogEntry.NORMAL_LOG,"Server Layer: Server Shutting Down"));

			this.destroy();
		    } 
		    try {
			Thread.sleep(1000);
		    }
		    catch(InterruptedException e) {}
		}
	    }
	}
	catch(Throwable e) {
	    conn_handler.handleError(new ConnectionError("Server stopped running:"+ e.toString()));
	}
    }
}


class RequestScheduler implements Runnable {
    private ConnectionHandler conn_handler;
    private RequestHandler[] my_handlers;
    private int MAX_HANDLERS= 10;
    private int num_threads;
    private Server my_server;
    private Queue socket_queue;
    private Queue handler_queue;
    private Thread my_thread;
    
    public RequestScheduler(Server s, ConnectionHandler ch, int num_threads, int queue_size) {
	this.conn_handler= ch;
	this.num_threads= num_threads;
	this.my_server= s;
	socket_queue= new Queue(queue_size);
	handler_queue= new Queue(num_threads);
	my_handlers= new RequestHandler[num_threads];
	try {
	    for (int i=0; i<num_threads; i++) {
		my_handlers[i]= new RequestHandler(s,this,this.conn_handler, i+1);
		handler_queue.push(my_handlers[i]);
	    }
	}
	catch(QueueFullException e) {}
	my_thread= new Thread(this);
	my_thread.start();
    }

    public void handle(Socket s) {
	RequestHandler rh;

	/* THIS used to cause a race condition. Instead
	 * we just put the socket on the queue, and let the thread 
	 * take care of things 
	 */
    
	try {
	    socket_queue.push(s);
	}
	catch(Exception e) {}
    }

    public void free(RequestHandler rh) {
	try {
	    handler_queue.push(rh);
	    /* Notify Server */
	    my_server.connectionEnd(rh.getNumber());
	}
	catch(QueueFullException e) {
	}
	rh.stop();
    }
    
    public String[] threadsStatus() {
	String[] ret= new String[my_handlers.length];
	for (int i=0; i<ret.length; i++) 
	    ret[i]= my_handlers[i].getHost();
      
	return(ret);
    }

    public int getNumBusyThreads() {
	return(num_threads-handler_queue.size());
    }

    public int getQueueSize() {
	return(socket_queue.size());
    }

    public void run() {
	RequestHandler rh;
	Socket s;
	while(true) {
	    try {
		Thread.sleep(100);
		if (socket_queue.isEmpty())
		    continue;
		if (handler_queue.isEmpty())
		    continue;
		rh= (RequestHandler)handler_queue.pop();
		Socket sock= (Socket)socket_queue.pop();
		rh.start(sock);
		/* Notify the Server of a Connection Start */
		my_server.connectionStart(rh.getNumber(), sock.getInetAddress());
	    }
	    catch(InterruptedException e) {}
	    catch(Exception e) {}
	}
    }
	    
    
}

class RequestHandler implements Runnable {
    private Thread my_thread;
    private Socket my_socket;
    private RequestScheduler my_scheduler;
    private Server my_server;
    private int my_number;
    private ConnectionHandler conn_handler;
    private boolean initialized= false;
    private boolean is_running= false;
    
    public RequestHandler(Server s, RequestScheduler rs, ConnectionHandler ch, int n) {
	my_server= s;
	my_scheduler= rs;
	conn_handler= ch;
	my_number= n;
	my_thread= new Thread(this);
    }

    public int getNumber() {
	return(my_number);
    }

    public void start(Socket s) {
	try {
	    /* This is in case the Handler is popped off the queue BEFORE
	     * it has a chance to be stopped!
	     */
	    while(is_running) {
		Thread.sleep(100);
	    }
	}
	catch(InterruptedException e) {}
	my_socket= s;
	if (!initialized) {
	    my_thread.start();
	    initialized= true;
	} else {
	    my_thread.resume();
	}
	is_running=true;
    }

    public void stop() {
	is_running= false;
	my_thread.suspend();
    }

    public String getHost() {
	if (this.is_running)
	    return(my_socket.getInetAddress().getHostName());
	else
	    return("FREE");
    }

    public boolean isRunning() {
	return(is_running);
    }
	    
    public void run() {
	while(true) {
	    Connection c= null;
	    try {
		my_socket.setSoTimeout(my_server.getTimeout());
		c= my_server.newConnection(my_socket);
		conn_handler.handleConnection(c);
	    }
	    catch(ConnectionException e) {
		conn_handler.handleError(new ConnectionError("couldn't create a Connection object"));
	    }
	    catch(SocketException se) {
		/* couldn't set timeout */
	    }
	    catch(InterruptedIOException e) {
		/* Time out */
	    }

	    /* Close the Connection */
	    if (c!= null)
		c.close();

	    c= null;
      
	    my_scheduler.free(this);
	}
    }
}

class RequestRecord {
    private int ip_hash;
    private InetAddress my_inet;
    private int number_hits;

    public RequestRecord(InetAddress ia) {
	my_inet= ia;
	ip_hash= ia.hashCode();
	number_hits= 1;
    }

    public synchronized void hit() {
	number_hits+= 1;
    }

    public int getNumHits() {
	return(number_hits);
    }

    public InetAddress getInetAddress() {
	return(my_inet);
    }
    
    public int getValue() {
	return(ip_hash);
    }
}


class RequestRecordList {
    protected RequestRecord my_record;
    protected RequestRecordList next;

    public RequestRecordList() {
	/* First record always kept at NULL! */
	my_record= null;
	next= null;
    }

    public void insert(RequestRecord rr) {
	RequestRecordList rrl= this;
	
	while(rrl.next!=null) {
	    rrl= rrl.next;
	}
	rrl.next= new RequestRecordList();
	rrl.next.my_record= rr;
	
    }

    public void remove(RequestRecord rr) {
	RequestRecordList rrl= this;

	while (rrl.next!= null) {
	    if (rrl.next.my_record == rr) {
		rrl.next= rrl.next.next;
		return;
	    }
	}
    }

    public void remove(int hash) {
	RequestRecordList rrl= this;

	while(rrl.next!= null) {
	    if (rrl.next.my_record.getValue() == hash) {
		rrl.next= rrl.next.next;
		return;
	    }
	}
    }
    
    public RequestRecord find(int value) {
	RequestRecordList rrl= this;

	while(rrl.next!= null) {
	    if (rrl.next.my_record.getValue()== value) {
		return(rrl.next.my_record);
	    }
	    rrl= rrl.next;
	}
	return(null);
    }

}


class RequestValidator {
    private Server my_server;
    private RequestRecordList black_list;
    private RequestRecordList suspect_list;
    private int max_hits;
    
    public RequestValidator(Server s, int mh) {
	my_server= s;
	this.max_hits= mh;
	black_list= new RequestRecordList();
	suspect_list= new RequestRecordList();
    }

    public void blackList(InetAddress ia) {
	black_list.insert(new RequestRecord(ia));
	my_server.blackList(ia);
    }

    public void clearRecord(InetAddress ia) {
	black_list.remove(ia.hashCode());
	my_server.blackListRemove(ia);
    }

    protected RequestRecordList getBlackList() {
	return(black_list);
    }
    
    public boolean validate(Socket s) {
	/* If BlackListed, refuse connection */
	RequestRecord rr;
	int hc= s.getInetAddress().hashCode();
	// System.out.println("LOOKING in blacklist");
	if (black_list.find(hc) != null)
	    return(false);
	// System.out.println("LOOKING in suspect list");
	rr= suspect_list.find(hc);
	if (rr==null) {
	    // System.out.println("NOT in suspect list");
	    suspect_list.insert(new RequestRecord(s.getInetAddress()));
	} else {
	    // System.out.println("in suspect list");
	    rr.hit();
	    if (rr.getNumHits() >= this.max_hits) {
		// System.out.println("BLACKLISTING");
		black_list.insert(rr);
		my_server.blackList(rr.getInetAddress());
		suspect_list.remove(rr);
	    }
	}
	return(true);
    }

}

