/* Electronic Voting Project
 * Server Interface
 * written by Ben Adida (ben@mit.edu)
 */

/* UPDATED to AWT 1.1 */

package evote.network.server;

import java.awt.*;
import java.awt.event.*;
import evote.network.server.*;
import evote.about.*;


/**
 * The AWT 1.1 Server Interface
 *
 * This is the default server interface which is a ConnectionListener and 
 * a BlackListListener.
 */
public class ServerInterface extends Frame implements ActionListener,ItemListener,WindowListener,BlackListListener,ConnectionListener {
  private Thread my_thread;
  private Server my_server;
  private List black_list, thread_list;
  private TextField threads_used_field, queue_size_field;
  private TextField status_field;
  private Button shutdown_button;
  private boolean shutdown_in_progress= false;

  /* Menus */
  /* Server Menu */
  private Menu server_menu;
  private MenuItem about_item, shutdown_item;
  
  /* Thread Menu */
  private Menu thread_menu;
  private MenuItem thread_info_item, thread_disconnect_item;

  /* BlackListed Item Menu */
  private Menu blacklisteditem_menu;
  private MenuItem blacklisted_info_item, blacklisted_reenable_item;

  /** 
   * The Server Interface constructor takes a name, and a Server,
   * which it will use when reenabling internet addresses, or shutting down
   *
   * @argument name The name of the Server Interface
   * @argument s The server that this interface is for
   */
  public ServerInterface(String name, Server s) {
    super(name);
    my_server= s;

    this.addWindowListener(this);
    
    /* my_thread= new Thread(this); */

    initGUI();
    this.pack();
    this.show();
    /* my_thread.start(); */
    //System.out.println("SERVER INTERFACE SHOULD COME UP");
  }

  private void initGUI() {
    
    /* Menus */
    MenuBar my_menubar= new MenuBar();

    server_menu= new Menu("Server");
    about_item= new MenuItem("About the Server");
    about_item.addActionListener(this);
    shutdown_item= new MenuItem("Shutdown");
    shutdown_item.addActionListener(this);
    server_menu.add(about_item);
    server_menu.add(shutdown_item);

    my_menubar.add(server_menu);

    thread_menu= new Menu("Thread");
    thread_info_item= new MenuItem("About this Thread");
    thread_info_item.addActionListener(this);
    thread_disconnect_item= new MenuItem("Disconnect");
    thread_disconnect_item.addActionListener(this);
    thread_menu.add(thread_info_item);
    thread_menu.add(thread_disconnect_item);
    
    my_menubar.add(thread_menu);
    
    blacklisteditem_menu= new Menu("Blacklisted Address");
    blacklisted_info_item= new MenuItem("About this Address");
    blacklisted_info_item.addActionListener(this);
    blacklisted_reenable_item= new MenuItem("Reenable this Address");
    blacklisted_reenable_item.addActionListener(this);
    blacklisteditem_menu.add(blacklisted_info_item);
    blacklisteditem_menu.add(blacklisted_reenable_item);
    
    my_menubar.add(blacklisteditem_menu);

    /* Disable the specific menus for now */
    thread_menu.setEnabled(false);
    blacklisteditem_menu.setEnabled(false);

    this.setMenuBar(my_menubar);

    /* Other components of the GUI */
    
    status_field= new TextField();
    status_field.setText("Server running.");
    threads_used_field= new TextField(4);
    queue_size_field= new TextField(6);
    
    status_field.setEditable(false);
    threads_used_field.setEditable(false);
    queue_size_field.setEditable(false);
    
    shutdown_button= new Button("Shutdown Server");
    shutdown_button.addActionListener(this);
    
    black_list= new List();
    black_list.addItemListener(this);
    thread_list= new List();
    thread_list.addItemListener(this);

    /* Layout */

    this.setLayout(new BorderLayout());

    Panel top= new Panel();
    top.setLayout(new GridLayout(2,1));
    Panel top_bottom= new Panel();
    Panel thread_panel= new Panel();
    Panel queue_panel= new Panel();
    
    thread_panel.setLayout(new GridLayout(1,2));
    queue_panel.setLayout(new GridLayout(1,2));
    
    thread_panel.add(new Label("Threads busy:",Label.CENTER));
    thread_panel.add(threads_used_field);
    
    queue_panel.add(new Label("Queue size:",Label.CENTER));
    queue_panel.add(queue_size_field);
    
    top_bottom.add(thread_panel);
    top_bottom.add(queue_panel);
    top.add(status_field);
    top.add(top_bottom);

    Panel center= new Panel();
    Panel center_top= new Panel();
    Panel center_bottom= new Panel();

    center_top.setLayout(new GridLayout(1,2));
    center_top.add(new Label("Threads:",Label.CENTER));
    center_top.add(new Label("Blacklisted Addresses:",Label.CENTER));
    
    center_bottom.setLayout(new GridLayout(1,2));
    center_bottom.add(thread_list);
    center_bottom.add(black_list);

    center.setLayout(new BorderLayout());
    center.add("North",center_top);
    center.add("Center",center_bottom);

    Panel bottom= new Panel();
    bottom.add(shutdown_button);
    
    this.add("North",top);
    this.add("Center",center);
    this.add("South",bottom);

    setNumThreadsUsed();
    setNumThreadsQueued();

    String[] threads_status= my_server.getAllThreadsStatus();
    for (int i=0; i<threads_status.length; i++)
      thread_list.addItem("Thread #" + (i+1) + ": --");

  }

  private void setThreadStatus(int n, String s) {
    n= n-1;
    boolean sel= false;
    if (thread_list.isIndexSelected(n))
      sel= true;
    thread_list.replaceItem("Thread #" + (n+1) + ": " + s, n);
    if (sel) {
      thread_list.select(n);
      thread_menu.setEnabled(true);
    }
  }

  private void setNumThreadsUsed() {
    threads_used_field.setText("" + my_server.getNumBusyThreads());
  }

  private void setNumThreadsQueued() {
    queue_size_field.setText("" + my_server.getQueueSize());
  }

  /**
   * This method is called when an element is added
   * to the Black List.
   *
   * @argument ble the Black list event
   */
  public void addBlackListedElement(BlackListEvent ble) {
    //System.out.println("INTERFACE: received element to black list");
    black_list.addItem(ble.getInetAddress().getHostName());
  }
  
  /** 
   * This method is called when an element is removed
   * from the black list.
   *
   * @argument ble the Black list event.
   */
  public void removeBlackListedElement(BlackListEvent ble) {
    black_list.remove(ble.getInetAddress().getHostName());
  }

  /**
   * This method is called when the black list is cleared
   *
   * @argument ble the Black list event
   */
  public void clearBlackListedElements(BlackListEvent ble) {
    black_list.removeAll();
  }

  /**
   * This method is called when a connection is started
   *
   * @argument ce the connection event
   */
  public void connectionStarted(ConnectionEvent ce) {
    //System.out.println("INTERFACE: got a connection start");
    this.setThreadStatus(ce.getThreadNumber(),ce.getInetAddress().toString());
    this.setNumThreadsUsed();
  }

  /**
   * This method is called when a connection is ended
   *
   * @argument ce the connection event
   */
  public void connectionEnded(ConnectionEvent ce) {
    this.setThreadStatus(ce.getThreadNumber(),"--");
    this.setNumThreadsUsed();
  }


  /**
   * This method is called when a connection is queued
   *
   * @argument ce the connection event
   */
  public void connectionQueued(ConnectionEvent ce) {
    this.setNumThreadsQueued();
  }

  /**
   * This method is called when a connection is dequeued
   *
   * @argument ce the connection event
   */
  public void connectionDequeued(ConnectionEvent ce) {
    this.setNumThreadsQueued();
  }
  
  /**
   * This destroys the interface
   */
  public void destroy() {
    /* This method is NOT called by the interface's own thread! */
    /* my_thread.stop();
     * my_thread=null;
     */
    this.setVisible(false);
    this.dispose();
  }

  public void abortShutdown() {
    my_server.abortShutdown();
    shutdown_in_progress= false;
    shutdown_button.setVisible(false);
    shutdown_button.setLabel("Shutdown");
    shutdown_button.setVisible(true);
    shutdown_item.setLabel("Shutdown Server");
    status_field.setText("Server is Running.");
  }

  public void shutdown() {
    //System.out.println("This should shut things down... ");

    if (shutdown_in_progress) {
      abortShutdown();
      return;
    }

    shutdown_in_progress= true;
    shutdown_button.setVisible(false);
    shutdown_button.setLabel("Abort Shutdown...");
    shutdown_button.setVisible(true);
    shutdown_item.setLabel("Abort Shutdown");
    status_field.setText("SERVER IS REFUSING CONNECTIONS... shutdown imminent");
    my_server.shutdown();
  }

  public void actionPerformed(ActionEvent e) {
    Object source= e.getSource();

    if ((source==shutdown_button)||(source==shutdown_item))
      shutdown();
    
    if (source==about_item) {
      new AboutBox();
    }
    
    if (source==thread_info_item) {
    }

    if (source==thread_disconnect_item) {
    }
    
    if (source==blacklisted_info_item) {
    }
    
    if (source==blacklisted_reenable_item) {
      my_server.clearRecord(black_list.getSelectedItem());
      blacklisteditem_menu.setEnabled(false);
    }      
  }

  public void itemStateChanged(ItemEvent e) {
    Object source= e.getSource();
    if (e.getStateChange()==ItemEvent.SELECTED) {
      if (source==black_list)
	blacklisteditem_menu.setEnabled(true);
      if (source==thread_list)
	thread_menu.setEnabled(true);
    }
    if(e.getStateChange()==ItemEvent.DESELECTED) {
      if (source==black_list)
	blacklisteditem_menu.setEnabled(false);
      if (source==thread_list)
	thread_menu.setEnabled(false);
    }
  }
   
  public void windowActivated(WindowEvent e) {
  }

  public void windowClosed(WindowEvent e) {
  }
  
  public void windowClosing(WindowEvent e) {
    shutdown();
  }

  public void windowDeactivated(WindowEvent e) {
  }

  public void windowDeiconified(WindowEvent e) {
  }

  public void windowIconified(WindowEvent e) {
  }

  public void windowOpened(WindowEvent e) {
  }
 
}
