/*
 * QuestionPanel.java
 * Last Revised:  13 May 97
 * Randy L. Milbert <rmilbert@mit.edu>
 */

package evote.question;

import evote.util.quick_message.QuickMessage;
import evote.election.*;
import evote.lang.*;

import java.awt.*;
import java.awt.event.*;

/**
 *  The QuestionPanel panel displays one question from the election.
 *  Whether or not the question allows write-ins is displayed as are
 *  the valid choices.
 */

public class MultipleQuestionPanel extends QuestionPanel implements ItemListener {

  private	Button			prev_button;
  private	Button			next_button;
  private	Button			done_button;
  private	int				num_question;
  private	int				num_choices;
  

  private java.awt.Choice[] choices;
  private TextField[] writeInFields;
  
  private Panel grandparent;
  private	QuestionListViewer parent;
  private	QuestionList election;
  private	evote.lang.MultipleChoice choice;

  /**
   *  The constructor takes five objects as arguments.  The ElectionViewer
   *  panel necessary for CardLayout.  The Dimension object specifies the
   *  height and width of the panel.  The Election object contians the
   *  text of the question and the text of the choices.  The choice object
   *  is altered when the user makes his or her selections.  The num_question
   *  variable is the question number of the election that this panel is
   *  responsible for.
   */
  
  public MultipleQuestionPanel(QuestionListViewer parent, QuestionList election, MultipleChoice choice, int num_question) {
    super();
    
    this.parent			= parent;
    this.election		= election;
    this.num_question	= num_question;
    this.choice			= choice;
	  
    setBackground(Color.lightGray);
	  
    Panel top_panel= new Panel();
    Panel question_panel= new Panel();
    Panel bottom_panel= new Panel();
    Panel label_panel= new Panel();
	  
    Question currentQuestion= election.getQuestion(num_question);
    int num_rows= this.choice.getNumChoices(num_question);

    // If we allow write-ins, then we need the 
    // space to put the text fields
    // we add one here for the question!
    if (currentQuestion.allowsWriteIns()) {
      question_panel.setLayout(new GridLayout(num_rows, 2));
    } else {
      question_panel.setLayout(new GridLayout(num_rows,1));
    }

    label_panel.setLayout(new GridLayout(num_rows,1));
    
    this.setLayout(new BorderLayout());
    this.add("North", top_panel);
    this.add("Center", question_panel);
    this.add("West", label_panel);

    bottom_panel.setLayout(new FlowLayout());
		
    num_choices			= currentQuestion.getNumAnswers();

    choices= new java.awt.Choice[num_rows];
    writeInFields= new TextField[num_rows];

    for (int counter=0; counter<num_rows; counter++) {
      choices[counter]= new java.awt.Choice();
      choices[counter].addItemListener(this);
      /* Add this to disable all but first */
      if (counter>0)
	choices[counter].setEnabled(false);
      label_panel.add(new Label("CHOICE #" + (counter+1)));
    }

    Label question_label= new Label(currentQuestion.getQuestion());
    // question_panel.add(question_label);
    top_panel.add(question_label);
	
    String[] answers= currentQuestion.getAnswers();
    for (int i = 0; i < num_rows; i++) {
      choices[i].addItem("----");
      for (int j= 0; j < num_choices; j++) {
	choices[i].addItem(answers[j]);
      }
      Panel helperPanel= new Panel();
      helperPanel.add(choices[i]);
      question_panel.add(helperPanel);
      if (currentQuestion.allowsWriteIns()) {
	choices[i].addItem("*WRITE IN*");
	helperPanel= new Panel();
	writeInFields[i]= new TextField(15);
	writeInFields[i].setEnabled(false);
	helperPanel.add(writeInFields[i]);
	question_panel.add(helperPanel);
      }
    }
		
    if (num_question != 0) {
      prev_button	= new Button(" < Previous");
      bottom_panel.add(prev_button);
      prev_button.addActionListener(this);
    }
		
    if (num_question == election.getNumQuestions() - 1) {
      done_button	= new Button("VOTE");
      bottom_panel.add(done_button);
      done_button.addActionListener(this);
    } else {
      next_button	= new Button("Next > ");
      bottom_panel.add(next_button);
      next_button.addActionListener(this);
    }

    this.add("South",bottom_panel);
    parent.add("QuestionPanel " + num_question, this);
  }

  public void actionPerformed(ActionEvent event) {
    Object source = event.getSource();
		
    if (source == next_button || source == done_button) {

      for (int i=0; i<choice.getNumChoices(num_question); i++) {
	// stop at first empty choice
	if (choices[i].getSelectedIndex()==0) {
	  for (int j=i; j<choice.getNumChoices(num_question); j++)
	    choice.setChoice(num_question+1, j+1, -1);
	  break;
	}

	if ((election.getQuestion(num_question).allowsWriteIns()) &&
	    (choices[i].getSelectedIndex() == (choices[i].getItemCount()-1)))
	  choice.setChoiceWriteIn(num_question+1, i+1, writeInFields[i].getText());
	else
	  choice.setChoice(num_question+1, i+1, choices[i].getSelectedIndex()-1);
      }

      if (source == next_button)
	((CardLayout)parent.getLayout()).next(parent);

      if (source == done_button) {
	parent.done();
      }
    } 
    
    if (source == prev_button) {
      ((CardLayout)parent.getLayout()).previous(parent);
    }

  }

  public void itemStateChanged(ItemEvent event) {
    Object source = event.getSource();
    
    int which_one, new_selection;

    which_one= -1;
    new_selection=-1;

    for (int i=0; i<choices.length; i++) {
      if (source == choices[i]) {
	which_one= i;
	new_selection= ((java.awt.Choice)source).getSelectedIndex();

	if (election.getQuestion(num_question).allowsWriteIns()) {
	  if (new_selection == (election.getQuestion(num_question).getNumAnswers() + 1)) {
	    writeInFields[i].setEnabled(true);
	  } else {
	    writeInFields[i].setEnabled(false);
	  }
	}
      } else {
	// prevent twice the same thing
	if ((choices[i].getSelectedIndex() > 0) && (choices[i].getSelectedIndex() < (election.getQuestion(num_question).getNumAnswers() + 1))) {
	  if (choices[i].getSelectedIndex() == ((java.awt.Choice)source).getSelectedIndex()) {
	    ((java.awt.Choice)source).select(0);
	  }
	}
      }
    }

    /* Now disable all the ones that need to be disabled
     */
    if (which_one > -1) {
      if (new_selection==0) {
	/* Disable all of the other ones */
	for(int i=which_one+1; i<choices.length; i++) {
	  choices[i].select(0);
	  choices[i].setEnabled(false);
	}
      } else {
	if ((which_one + 1) < choices.length)
	  choices[which_one+1].setEnabled(true);
      }
    }       
  }
	  
}
