package evote.voter;

import java.applet.Applet;
import java.io.*;
import java.math.BigInteger;
import java.net.*;
import java.security.*;

import evote.commissioner.complaints.*;
import evote.crypto.*;
import evote.election.*;
import evote.election.viewer.*;
import evote.lang.*;
import evote.network.message.*;
import evote.network.server.*;
import evote.util.constants.*;
import evote.util.genrand.*;
import evote.util.quick_message.*;
import evote.question.*;


/**
  This is the voting "booth" applet that the voter will download and use.  It
  read in an election object and then creates a connection to first the admin
  server and then the anonymizer.
  Revelant numbers, messages and errors will be shown in the appropriate areas
  of the applet.
 */
public class Booth extends Applet implements RandomDataRequestor {

    private static final int NUM_PREFERENCES= 3;

    Election election;
    evote.lang.Choice myChoice;
    VoteSender myVoteSender;

    private GenRand RNG;
    private QuestionList myQuestionList;
    private byte[] questionListSignature;

    private String paramName, paramPassword;
    private VoterIdentityObject myIdentity;

    private static final String VOTER_ELECTION_FILE= "Voter.election";

    public void init() {
	// System.out.println("CHECK -- reload");
	paramName= getParameter("username");
	paramPassword= getParameter("password");
    
	try {
	    /* Load up the Election Object */
	    URL voterConfigURL= new URL(getCodeBase(),VOTER_ELECTION_FILE);
	    InputStream is= voterConfigURL.openStream();
	    ObjectInputStream ois= new ObjectInputStream(is);
	    election= (Election)ois.readObject();
	    Constants.init(election);
	} catch(Exception exc) {
	    /* Do something drastic if an exception is encountered
	     */
	    System.out.println("Error loading up election object: " + exc.toString());
	    exc.printStackTrace();
	}

    }

    public void setup() {
	RNG= new GenRand();
	RNG.seed();
    }

    private void setIdentity(String name, String password) {
	myIdentity= new VoterIdentityObject(name, password);
    }

    public void randomDataGenerated() {
    }    

    // Set the choice for 
    // - the qln (question list number)
    // - the qn (question number)
    public void setChoice(int qln, int qn, int ch) {
	try {
	    // System.out.println("choice being set for " + qln  + " - " + qn + " as " + ch);
	    Question currentQuestion;
      
	    for (int i=0; i<myQuestionList.getNumQuestions(); i++) {
		currentQuestion= myQuestionList.getQuestion(i);
		if ((currentQuestion.getQuestionListID() == qln) &&
		    (currentQuestion.getQuestionID() == qn)) {
		    myChoice.setChoice(i, ch);
		    break;
		}
	    }
	} catch (Exception exc) {
	    exc.printStackTrace();
	}
    }

    // Set the choice for 
    // - the qln (question list number)
    // - the qn (question number)
    // - the pref position
    public void setChoice(int qln, int qn, int pref, int ch) {
	try {
	    // System.out.println("choice being set for " + qln  + " - " + qn + " - " + pref + " as " + ch);
	    Question currentQuestion;
      
	    for (int i=0; i<myQuestionList.getNumQuestions(); i++) {
		currentQuestion= myQuestionList.getQuestion(i);
		if ((currentQuestion.getQuestionListID() == qln) &&
		    (currentQuestion.getQuestionID() == qn)) {
		    // System.out.println("question #" + i + ", pref " + pref + " - " + ch);
		    ((MultipleChoice)myChoice).setChoice(i, pref, ch);
		    break;
		}
	    }
	} catch (Exception exc) {
	    exc.printStackTrace();
	}
    }



    // Set the choice for 
    // - the qln (question list number)
    // - the qn (question number)
    public void setChoiceWriteIn(int qln, int qn, String ch) {
	Question currentQuestion;

	for (int i=0; i<myQuestionList.getNumQuestions(); i++) {
	    currentQuestion= myQuestionList.getQuestion(i);
	    if ((currentQuestion.getQuestionListID() == qln) &&
		(currentQuestion.getQuestionID() == qn)) {
		myChoice.setChoiceWriteIn(i, ch);
		break;
	    }
	}
    }

    // Set the choice for 
    // - the qln (question list number)
    // - the qn (question number)
    // - the pref
    public void setChoiceWriteIn(int qln, int qn, int pref, String ch) {
	Question currentQuestion;

	for (int i=0; i<myQuestionList.getNumQuestions(); i++) {
	    currentQuestion= myQuestionList.getQuestion(i);
	    if ((currentQuestion.getQuestionListID() == qln) &&
		(currentQuestion.getQuestionID() == qn)) {
		((MultipleChoice)myChoice).setChoiceWriteIn(i, pref, ch);
		break;
	    }
	}
    }

    public int getNumPreferences(int questionNum) {
	return(myQuestionList.getQuestion(questionNum).getNumAnswers());
    }

    public Choice getChoiceObject() {
	return(myChoice);
    }

    public boolean doLogon(String name, String passwd) {
    
	this.setIdentity(name, passwd);

	// Create a Logon Message
	LogonMessage lm= new LogonMessage(myIdentity);
    
	try {
	    // Connect to the Admin
	    SecureConnection adminConnection = new SecureConnection(Constants.adminServer, Constants.adminReceivePort, RNG, Constants.adminKeys);

	    // send logon message
	    adminConnection.sendMessage(lm);
      
	    // get response
	    Message response= adminConnection.receiveMessage();

	    // if it's a question list, cool!
	    if (response instanceof QuestionListMessage) {
		// verify the signature on the question List
		QuestionListMessage qlm= (QuestionListMessage)response;
		QuestionList myQL= qlm.getQuestionList();

		myQuestionList= myQL;
		questionListSignature= qlm.getSignature();

		switchToQuestions();
		return(true);
	    }

	    // if it's an error
	    if (response instanceof ErrorMessage) {
		throw(new Exception("could not log on"));
	    }

	    return(false);
	} 
	catch(Exception exc) {
	    exc.printStackTrace();
	    return(false);
	}
    }

    /**
       This switches the screen to Question Answering Mode
    */
    private void switchToQuestions() {
	myChoice= new evote.lang.Choice(myQuestionList.getNumQuestions());
	/*
	  myChoice= new evote.lang.MultipleChoice(myQuestionList.getNumQuestions());
	  for (int i=0; i<myQuestionList.getNumQuestions(); i++) {
	  ((MultipleChoice)myChoice).setQuestionSize(i, NUM_PREFERENCES);
	  }	
	*/
    }

    public int[] getQuestionListNumbers() {
	int[] test= {1, 2, 3, 4};
	//return(test);
    
	int[] qlnums= new int[myQuestionList.getNumQuestions()];
	int max= 0;
	// System.out.println("qlnums.length: " + qlnums.length);
	for (int i=0; i<qlnums.length; i++) {
	    qlnums[i]= myQuestionList.getQuestion(i).getQuestionListID();
	    if (qlnums[i]>max)
		max=qlnums[i];
	}
    
	// create an array of booleans
	// to check if that particular questionlist is there
	boolean[] qlnums_present= new boolean[max+1];
	// System.out.println("qlnums_present.length: " + qlnums_present.length);
	for (int i=0; i<qlnums_present.length; i++) 
	    qlnums_present[i]= false;
    
	// using the boolean array, get a list of unique
	// question list ids
	int num_qlists=0;
	for (int i=0; i<qlnums.length; i++) {
	    if (!qlnums_present[qlnums[i]])
		num_qlists+=1;
	    qlnums_present[qlnums[i]]= true;
	}
    
	int[] questionListNumbers= new int[num_qlists];
	int count=0;
	for (int i=0; i<qlnums_present.length; i++) {
	    if (qlnums_present[i]) {
		questionListNumbers[count]= i;
		count+=1;
	    }
	}
    
	return(questionListNumbers);
    }

    public synchronized boolean goVote() {
	try {
	    if (myVoteSender == null) {
		myVoteSender= new VoteSender(myIdentity, RNG);
      
		myVoteSender.setChoice(myChoice);
		//System.out.println("Setting the Question List!");
		myVoteSender.setQuestionList(myQuestionList, questionListSignature);
	    }
      
	    return(myVoteSender.sendVote());
	} catch (Exception exc) {
	    exc.printStackTrace();
	    return(false);
	}
    }

    public void reset() {
	myVoteSender= null;
	myQuestionList= null;
	questionListSignature=null;
    }

    public void gotoError() {
    }    

    public void done() {
    }

} 

