/*
 * This class is responsible for sending the vote
 * It is part of the Booth rearchitecturing by Ben Adida (ben@mit.edu)
 *
 * This is the piece that takes care of sending the vote
 */

package evote.voter;

import java.net.*;
import java.io.*;

import evote.commissioner.complaints.*;
import evote.crypto.*;
import evote.lang.*;
import evote.network.message.*;
import evote.network.server.*;
import evote.util.constants.*;
import evote.util.genrand.*;
import evote.question.*;

public class VoteSender {
  
  private boolean choiceReceived= false;
  private boolean adminSent= false;
  private boolean adminReceived= false;
  private boolean anonSent= false;
  private boolean anonReceived= false;
  private boolean questionListSet= false;
 
  private Choice myChoice;
  private Vote myVote;
  private Vote adminSendVote;
  private Vote adminReceiveVote;
  private Vote anonSendVote;
  private VoterIdentityObject myIdentity;
  private GenRand myRandom;

  private RsaBlind blinder;
  private CommitmentKeys commitmentKeys;

  private SecureConnection adminConnection, anonConnection, counterConnection;

  public VoteSender(VoterIdentityObject vido, GenRand RNG) {
    myIdentity= vido;
    myRandom= RNG;
  }

  public synchronized void setChoice(Choice c) {
    myChoice= c;
	
    try {
      myVote= new Vote(myChoice);
    } catch(Exception exc) {
      return;
    }

    packageVote();

    choiceReceived= true;
  }

  public synchronized void setQuestionList(QuestionList ql, byte[] signature) {
    if (!choiceReceived)
      return;
    if (questionListSet)
      return;

    myVote.setQuestionList(ql, signature);

    questionListSet= true;
  }

  public synchronized boolean sendVote() {
    if (!choiceReceived)
      return(false);


    System.out.println("sending to admin");
    // SEND stuff to ADMIN
    if (!adminSent)
      adminSend();
    if (!adminSent)
      return(false);
    

    System.out.println("receiving from admin");
    // RECEIVE stuff from ADMIN
    if (!adminReceived)
      adminReceive();
    if (!adminReceived)
      return(false);

    System.out.println("sending stuff to Anon");
    
    // SEND stuff to ANON
    if (!anonSent)
      anonSend();
    if (!anonSent)
      return(false);

    // RECEIVE stuff from ANON
    if (!anonReceived)
      anonReceive();
    if (!anonReceived)
      return(false);

    return(true);
  }

  private void packageVote() {
    // commit the vote
    ShaCommitment commit= new ShaCommitment(myRandom);
    commitmentKeys= commit.getKeys();
    myVote.commit(commit);
  }
  
  private void adminSend() {
    try {
      // Blind the vote
      blinder= new RsaBlind(Constants.adminKeys, myRandom);
      myVote.blind(blinder);

      adminSendVote= myVote.makeAdminSafe();
      
      VoteMessage message_to_admin= new VoteMessage(adminSendVote, myIdentity);

      if (adminConnection == null)
	adminConnection= new SecureConnection(Constants.adminServer, Constants.adminReceivePort, myRandom, Constants.adminKeys);

      adminConnection.sendMessage(message_to_admin);
      
      adminSent= true;
    } catch(Exception exc) {
      try {
	adminConnection.close();
      } catch (Exception exc2) {
      }

      adminConnection= null;
      adminSent= false;
    }
  }

  private void adminReceive() {
    try {
      Message message_from_admin= adminConnection.receiveMessage();

      // process the message if we receive one that contains
      // a vote with a signature
      if (message_from_admin instanceof ErrorMessage)
	throw(new Exception("Error: "));
      
      if (message_from_admin instanceof VoteMessage) {
	adminConnection.close();

	adminReceiveVote= ((VoteMessage)message_from_admin).getVote();

	myVote.getSignature(adminReceiveVote);

	myVote.unblind(blinder);

	// if vote is not verified
	if (!myVote.verifyAdminSign(new RsaEncryption(Constants.adminKeys)))
	  return;

	adminReceived= true;
      }
    } catch (Exception exc) {
      try {
	adminSent= false;
	adminReceived= false;
	if (adminConnection!=null)
	  adminConnection.close();
	adminConnection=null;
      } catch (Exception exc2) {
      }
    }
  }

  private void anonSend() {
    try {
      if (anonConnection == null)
	anonConnection= new SecureConnection(Constants.anonServer, Constants.anonReceivePort, myRandom, Constants.anonKeys);
      if (counterConnection == null)
	counterConnection= new SecureConnection(anonConnection, myRandom, Constants.counterKeys);
      
      anonSendVote= myVote.makeCounterSafe();

      //
      // SOMETHING NEEDS TO BE DONE HERE ABOUT QUESTIONLISTS
      //

      VoteMessage message_to_anon= new VoteMessage(anonSendVote);
      
      counterConnection.sendMessage(message_to_anon);

      anonSent= true;
    } catch (Exception exc) {
      try {
	if (counterConnection != null) {
	  counterConnection.close();
	  counterConnection= null;
	}
	if (anonConnection != null) {
	  anonConnection.close();
	  anonConnection= null;
	}
      } catch (Exception exc2) {
      }
    }
  }

  private void anonReceive() {
    try {
      Message message_from_anon= anonConnection.receiveMessage();
      
      if (message_from_anon instanceof OKMessage) {
	anonReceived= true;
	return;
      }
    } catch (Exception exc) {
      try {
	if (counterConnection != null) {
	  counterConnection.close();
	  counterConnection= null;
	}
	if (anonConnection != null) {
	  anonConnection.close();
	  anonConnection= null;
	}
      } catch (Exception exc2) {
      }
    }      
  }
}
