/* Electronic Voting Project
 * Mark A. Herschberg (hershey@mit.edu)
 */

package evote.crypto;

import java.math.BigInteger;
import java.security.*;
import java.util.Random;

import evote.util.constants.*;
import evote.util.genrand.*;

/**
  Performs RSA blinding (e.g. for use with blind signatures as listed in
  Bruce Schneier's "Applied Cryptography" (2nd ed) p 549.
  */
public class RsaBlind implements Blind {
  private RsaKeys keys;
  private BigInteger blindingFactor;
  private NoKeyException nke;
  
  /**
    Creates a RsaBlind object using the given RSA keys and a byte array as the
    seed for the blinding factor (created from the seed using SecureRandom).
    */
  public RsaBlind(RsaKeys givenKeys, byte[] seed) {
    keys = givenKeys;
    SecureRandom SRNG = new SecureRandom(seed);
    int len = (keys.getRsaN()).bitLength() - 1;
    do {
      blindingFactor = new BigInteger(len, SRNG);
    } while (blindingFactor.compareTo(keys.getRsaN()) >= 0);
  }
  
  /**
    Creates a RsaBlind object using the given RSA keys and a blinding factor
    created using the GenRand object.
    */
  public RsaBlind(RsaKeys givenKeys, GenRand RNG) {
    keys = givenKeys;
    int len = (keys.getRsaN()).bitLength() - 1;
    do {
      blindingFactor = new BigInteger(len, new SecureRandom(RNG.nextBytes(Constants.nextBytes)));
    } while (blindingFactor.compareTo(keys.getRsaN()) >= 0);
  }

  public RsaBlind(RsaKeys givenKeys, SecureRandom srand) {
    keys= givenKeys;
    int len= (keys.getRsaN()).bitLength() - 1;
    do {
      blindingFactor = new BigInteger(len, srand);
    } while (blindingFactor.compareTo(keys.getRsaN()) >= 0);
  }    
  
  /**
    Creates a RsaBlind object using the given RSA keys and blinding factor.
    */
  public RsaBlind(RsaKeys givenKeys, BigInteger bf) {
    keys = givenKeys;
    blindingFactor = bf;
  }
  
  /**
    Creates a RsaBlind object using the given e and n, and a byte array as the
    seed for the blinding factor (created from the seed using SecureRandom).
    */
  public RsaBlind(BigInteger pub, BigInteger modulus, byte[] seed) {
    keys = new RsaKeys(pub, modulus);
    SecureRandom SRNG = new SecureRandom(seed);
    int len = (keys.getRsaN()).bitLength() - 1;
    do {
      blindingFactor = new BigInteger(len, SRNG);
    } while (blindingFactor.compareTo(keys.getRsaN()) >= 0);
  }
  
  /**
    Creates a RsaBlind object using the given e and n, and a blinding factor
    created using the GenRand object.
    */
  public RsaBlind(BigInteger pub, BigInteger modulus, GenRand RNG) {
    keys = new RsaKeys(pub, modulus);
    int len = (keys.getRsaN()).bitLength() - 1;
    do {
      blindingFactor = new BigInteger(len, new SecureRandom(RNG.nextBytes(Constants.nextBytes)));
    } while (blindingFactor.compareTo(keys.getRsaN()) >= 0);
  }
  
  /**
    Creates a RsaBlind object using the given e, n, and blinding factor.
    */
  public RsaBlind(BigInteger pub, BigInteger modulus, BigInteger bf) {
    keys = new RsaKeys(pub, modulus);
    blindingFactor = bf;
  }
  
  
  /**
    Creates a RsaBlind object using the given e, d, and n, and a byte array as
    the seed for the blinding factor (created from the seed using
    SecureRandom).
    */
  public RsaBlind(BigInteger pub, BigInteger priv, BigInteger modulus, byte[] seed) {
    keys = new RsaKeys(pub, priv, modulus);
    SecureRandom SRNG = new SecureRandom(seed);
    int len = (keys.getRsaN()).bitLength() - 1;
    do {
      blindingFactor = new BigInteger(len, SRNG);
    } while (blindingFactor.compareTo(keys.getRsaN()) >= 0);
  }
  
  /**
    Creates a RsaBlind object using the given e, d, and n, and a blinding
    factor created using the GenRand object.
    */
  public RsaBlind(BigInteger pub, BigInteger priv, BigInteger modulus, GenRand RNG) {
    keys = new RsaKeys(pub, priv, modulus);
    int len = (keys.getRsaN()).bitLength() - 1;
    do {
      blindingFactor = new BigInteger(len, new SecureRandom(RNG.nextBytes(Constants.nextBytes)));
    } while (blindingFactor.compareTo(keys.getRsaN()) >= 0);
  }
  
  /**
    Creates a RsaBlind object using the given e, d, n, and blinding factor.
    */
  public RsaBlind(BigInteger pub, BigInteger priv, BigInteger modulus, BigInteger bf) {
    keys = new RsaKeys(pub, priv, modulus);
    blindingFactor = bf;
  }
  
  /**
    Blinds the input by the blinding factor inherent in the RsaBlind object.
    Throws a NoKeyException if e or n is null.
    */
  public byte[] blind(byte[] input) throws Exception {
    BigInteger number = new BigInteger(input);  //force it positive 
    BigInteger blindedVote;
    
    if ( (keys.getRsaE() == null) || (keys.getRsaN() == null) ) {
      NoKeyException nke = new NoKeyException("Blind: e=" + (keys.getRsaE() == null ? "NULL" : (keys.getRsaE()).toString()) + " n=" + (keys.getRsaN() == null ? "NULL" : (keys.getRsaN()).toString()));
      throw(nke);
    }
    else {
      try {
	blindedVote = ((blindingFactor.modPow(keys.getRsaE(), keys.getRsaN())).multiply(number)).mod(keys.getRsaN());
      }
      catch (ArithmeticException exp) {
	System.out.println("You've got a problem when blinding");
	blindedVote = new BigInteger("-1");
      }
      return blindedVote.toByteArray();
    }
  }
  
  /**
    Unblinds the input using the blinding factor inherent in the RsaBlind
    object. Throws a NoKeyException if n is null.
    */
  public byte[] unblind(byte[] input) throws Exception {
    BigInteger number = new BigInteger(input);
    BigInteger unblindedVote;
    
    if (keys.getRsaN() == null) {
      NoKeyException nke = new NoKeyException("Unblind: n=" + (keys.getRsaN() == null ? "NULL" : (keys.getRsaN()).toString()));
      throw(nke);
    }
    else {
      try {
	unblindedVote = (number.multiply(blindingFactor.modInverse(keys.getRsaN()))).mod(keys.getRsaN());
      }
      catch (ArithmeticException exp) {
	System.out.println("You've got a problem when unblinding");
	unblindedVote = new BigInteger("-1");
      }
      return unblindedVote.toByteArray();
    }
  }
  
  /**
    Returns the public keys used for blinding.
    */
  public RsaKeys getRsaKeys() {
    return keys;
  }
  
  /**
    Returns the number used for blinding.
    */
  public BigInteger getBlindingFactor() {
    return blindingFactor;
  }
  
}

