/* Electronic Voting Project
 * Ben Adida (ben@mit.edu)
 */

/* This class is the raw choice in the election process
 * There can be write-ins.
 */

package evote.lang;

import java.io.*;
import evote.crypto.*;
import evote.lang.*;

/**
 * The Choice object
 *
 * The choice object encompasses all the selections of a voter
 * in a given election. It is simply a data structure that centralizes
 * all this information.
 * <p>
 * A choice can contain numbered choices, or write-ins.
 * <p>
 * The Choice object has questions indexed starting at 1.
 */
public class Choice implements Serializable,Cloneable {
  private int[] choices;
  private boolean[] is_write_in;
  private String[] write_in_data;

  /* For superclasses */
  protected Choice() {
  }

  /**
   * Create a Choice object of the given size
   *
   * @argument size The size of the choice, i.e. the number of questions.
   */
  public Choice(int size) {
    choices= new int[size];
    is_write_in= new boolean[size];
    write_in_data= new String[size];
    for (int i=0; i< size; i++) {
      is_write_in[i]= false;
      choices[i]= 0;
    }
  }

  /**
   * Get the number of questions of this choice
   *
   * @return The number of questions 
   */
  public int getSize() {
    return(choices.length);
  }

  /**
   * Set the answer to a given question to the given selection
   *
   * @argument choice_num The question number
   * @argument ch The option chosen for that question
   */
  public void setChoice(int choice_num, int ch) {
    is_write_in[choice_num]= false;
    choices[choice_num]= ch;
  }

  /**
   * Set the answer to a given question to a write-in that is provided
   * @argument choice_num The question number
   * @argument wi The write-in choice
   */
  public void setChoiceWriteIn(int choice_num, String wi) {
    is_write_in[choice_num-1]= true;
    write_in_data[choice_num-1]= wi;
  }

  /**
   * Gets the pre-set choice answer to a given question
   * 
   * @argument choice_num The question number
   * @return The chosen selection for that question
   * @exception IndexOutOfBoundsException
   *    This exception is thrown if the number of the question
   *    is out of bounds
   * @exception WriteInException
   *    This exception is thrown if the answer to this question
   *    is a write-in.
   */
  public int getChoice(int choice_num) throws IndexOutOfBoundsException,WriteInException {
    if (choice_num> choices.length)
      throw(new IndexOutOfBoundsException());

    if (is_write_in[choice_num-1])
      throw(new WriteInException());

    return(choices[choice_num-1]);
  }

  /**
   * This returns the String of the write-in answer for a given question
   * 
   * @argument choice_num The question number
   * @return the String of the write-in
   * @exception IndexOutOfBoundsException
   *    This is thrown if the number of the question is out of bounds.
   * @exception ChoiceException
   *    This is thrown if the answer to this question
   *    is a choice, and not a write-in
   */
  public String getWriteInChoice(int choice_num) throws IndexOutOfBoundsException,ChoiceException {
    if (choice_num> choices.length)
      throw(new IndexOutOfBoundsException());

    if (!is_write_in[choice_num-1])
      throw(new ChoiceException());

    return(write_in_data[choice_num-1]);
  }

  /**
   * This serializes the Choice object to an array of bytes
   *
   * @return The array of bytes
   * @exception Exception
   *    This is thrown if a problem comes up in serializing.
   */
  public byte[] serialize() throws Exception {
    byte[] ser_choice;
    ByteArrayOutputStream baos= new ByteArrayOutputStream();
    try {
      ObjectOutputStream oos = new ObjectOutputStream(baos);
      oos.writeObject(this);
      ser_choice= baos.toByteArray();
      return(ser_choice);
    }
    catch(IOException ie) {
      throw(new Exception("couldn't serialize"));
    }
  }

  /**
   * This re-creates a Choice object from an array of bytes
   * 
   * @argument b The array of bytes from which to recreate the Choice.
   * @exception Exception
   *   This is thrown if the bytes cannot be deserialized.
   */
  public static Choice deserialize(byte[] b) throws Exception {
    try {
      ByteArrayInputStream bais= new ByteArrayInputStream(b);
      ObjectInputStream ois= new ObjectInputStream(bais);
      Choice c= (Choice)ois.readObject();
      return(c);
    }
    catch(Exception e) {
      throw(new Exception("Format Error"));
    }
  }

}




