/* Electronic Voting Project
 * Connection class
 * written by Ben Adida (ben@mit.edu)
 */

package evote.network.server;

import java.io.*;
import java.net.*;

/**
 * The network connection abstraction.
 * A <pre>Connection</pre> object abstracts a network connection, whether 
 * it be on the server side, or on the client side. The client-side
 * constructor takes care of creating the actual socket connection to the 
 * indicated server, while the server-side constructor is called by the
 * Server class, and should not be called by other code.
 * <p>
 * Connections communicate with each other using the <pre>Message</pre> class.
 *
 * @see evote.network.server.Message
 */

public class Connection {
  protected Socket my_socket;
  protected ObjectInputStream ois;
  protected ObjectOutputStream oos;

  /**
   * constructs a connection from the client side.
   *
   * @argument server The String that represents the server host name.
   * @argument port The port number that the server is running on.
   * @return the created Connection.
   * @exception evote.network.server.ConnectionException
   *    If the connection cannot be made, this exception is thrown.
   */
  public Connection(String server, int port) throws ConnectionException {
    try {
      my_socket= new Socket(server,port);
      my_socket.setSoTimeout(60000);
    }
    catch(Exception e) {
      throw(new ConnectionException("Couldn't connect"));
    }
    initStreams();
  }

  public Connection(Socket s) throws ConnectionException{
    my_socket= s;
    initStreams();
  }

  public Connection() {
  }    

  private void initStreams() throws ConnectionException {
    try {
      oos= new ObjectOutputStream(my_socket.getOutputStream());
      ois= new ObjectInputStream(my_socket.getInputStream());
    }
    catch(Exception e) {
      throw(new ConnectionException("Couldn't connect"));
    }
  }
      
  /**
   * This sets the timeout on the connection, if such a timeout is wanted.
   * A timeout of 0 means no timeout.
   *
   * @argument to The number of milliseconds to timeout.
   */
  public void setTimeOut(int to) {
    // implemented only in Java 1.1, so we'll leave it alone for now
    //my_socket.setSoTimeout(to);
  }

  /**
   * This gets the InetAddr of the remote host that the connection
   * is connected to.
   *
   * @return The desired InetAddr
   */
  public InetAddress getInetAddress() {
    return(my_socket.getInetAddress());
  }

  /** 
   * This sends a Message through the Connection to the other side.
   *
   * @argument m the message to be sent.
   * @exception evote.network.server.ConnectionException
   *    If the message cannot be sent for whatever reason, this exception 
   *    is thrown
   * @see evote.network.server.Message
   */
  public void sendMessage(Message m) throws ConnectionException{
    try {
      oos.writeObject(m);
    }
    catch(Exception e) {
      throw(new ConnectionException("Message could not be sent: "+ e.toString()));
    }
  }

  /** 
   * This sends a message m completely unaltered. The purpose of
   * this method becomes more apparent in SecureConnection
   *
   * @argument m The Message that is to be sent
   * @exception evote.network.server.ConnectionException
   *     If the message cannot be sent, this exception is thrown.
   */
  public void sendUnencryptedMessage(Message m) throws ConnectionException {
    this.sendMessage(m);
  }
  
  /**
   * This blocks until a Message is received by the connection, or until
   * the connection times out (given the timeout that was set
   *
   * @return The received Message
   * @exception evote.network.server.ConnectionException
   *     If the connection breaks, or cannot receive a message
   *     this exception is thrown
   * @see evote.network.server.Message
   */
  public Message receiveMessage() throws ConnectionException{
    try {
      return((Message)ois.readObject());
    }
    catch(Exception e) {
      throw(new ConnectionException("Message could not be read: "+ e.toString()));
    }
  }

  /** 
   * This closes the connection
   */
  public void close() {
    try {
      ois.close();
      oos.close();
      my_socket.close();
    }
    catch(Throwable e) {}
  }
}
  

  
