/* Electronic Voting Project
 * written by Ben Adida(ben@mit.edu)
 * A Crypt Message is a wrapper message class for sending
 * encrypted messages 
 */

package evote.network.server;

import java.io.*;
import evote.crypto.*;
import java.math.*;


/**
 * A message that contains another encrypted message.
 *
 * A <pre>CryptMessage</pre> is a <pre>Message</pre> that contains another
 * encrypted <pre>Message</pre>. This is used for abstracting encrypted 
 * communication through the usual Message interface.
 * <p>
 * A CryptMessage as defined here contains another message encrypted with 
 * some crypto algorithm, whose key is included in the CryptMessage as 
 * a session key. This session key is itself encrypted in some way. It is
 * obviously preferable to make the crypto algorithm that encrypts the Message
 * as fast as possible, since a Message may be large, and the crypto algorithm
 * that encrypts the session key as secure as possible.
 * <p>
 * The CryptMessage also contains information for the MAC to ensure message
 * integrity.
 */
public class CryptMessage extends Message {
  private byte[] mac;
  private byte[] session_key;
  private byte[] stuff;
  private byte[] commit_keys;

  /**
   * constructs a new CryptMessage on top of the original Message, performing
   * all of the necessary encryption and MAC creation.
   *
   * @argument m The message to be encrypted
   * @argument message_encrypted The actual message encrypter.
   * @argument server_encrypter The session key encrypter
   * @argument integrity_checker The hashing function that's used to check integrity.
   * @return The desired CryptMessage
   * @exception evote.crypto.CryptoException 
   *     If a cryptographic problem happens during CryptMessage creation,
   *     this exception is thrown
   * 
   * @see evote.crypto.EncryptClass
   * @see evote.crypto.IntegrityClass
   */
  public CryptMessage(Message m, EncryptClass message_encrypter, EncryptClass server_encrypter, IntegrityClass integrity_checker) throws CryptoException {
    super("","");

    ObjectOutputStream oos;
    ByteArrayOutputStream baos;
    byte[] unencrypted_stuff;
    byte[] unencrypted_mac;
	
    try {
      /* Serialize the object */
      baos= new ByteArrayOutputStream();
      oos= new ObjectOutputStream(baos);
      
      oos.writeObject(m);
      unencrypted_stuff= baos.toByteArray();

      oos.close();
      baos.close();

      /* Serialize the Commitment Keys */
      baos= new ByteArrayOutputStream();
      oos= new ObjectOutputStream(baos);
      
      oos.writeObject(integrity_checker.getKeys());
      commit_keys= baos.toByteArray();
      commit_keys= message_encrypter.encrypt(commit_keys);

      /* Make the Mac */
      unencrypted_mac= integrity_checker.makeChecksum(unencrypted_stuff);
      this.mac= message_encrypter.encrypt(unencrypted_mac);

      /* Encrypt the Object */
      this.stuff= message_encrypter.encrypt(unencrypted_stuff);
      

      //System.out.println("setting session key");
      /* Encrypt the Session Key */
      this.session_key= server_encrypter.encrypt(message_encrypter.getEncryptionKey().toByteArray());
      //System.out.println("set session key");


      oos.close();
      baos.close();
    }
    catch(Exception e) {
      throw(new CryptoException("couldn't create CryptoObject: "+ e.toString()));
    }
  }

  /**
   * Returns the bytes of the encrypted session key
   *
   * @return the bytes of the encrypted session key
   */
  public byte[] getEncryptedSessionKey() {
    return(session_key);
  }
  
  /** 
   * returns the bytes of the encrypted commitment keys
   *
   * @return the bytes of the encrypted commitment keys
   */
  public byte[] getEncryptedCommitKeys() {
    return(commit_keys);
  }

  /**
   * Obtain the actual commitment keys
   * If any problem happens in decrypting the commitment keys,
   * a null is returned.
   *
   * @argument message_encrypter the encrypter that can decrypt the commitment keys
   * @return the CommitmenKeys object.
   */
  public CommitmentKeys getCommitmentKeys(EncryptClass message_encrypter) {
    try {
      ObjectInputStream ois;
      ByteArrayInputStream bais= new ByteArrayInputStream(message_encrypter.decrypt(commit_keys));
      
      ois= new ObjectInputStream(bais);
      
      CommitmentKeys ck= (CommitmentKeys)ois.readObject();
      
      return(ck);
    }
    catch(Exception e) {
      return(null);
    }
  }

  /**
   * Obtain the original Message that was encrypted.
   *
   * @argument encrypter The encrypter that can decrypt the message
   * @argument integrity_checker The integrity checker to verify the message
   * @return If all goes well, the original Message is returned.
   * @exception evote.crypto.CryptoException 
   *     If problems happen at decryption or verification, this exception is
   *     thrown.
   */
  public Message decryptAndVerify(EncryptClass encrypter, IntegrityClass integrity_checker) throws CryptoException {
    byte[] unencrypted_mac;
    byte[] unencrypted_stuff;
    ObjectInputStream ois;
    ByteArrayInputStream bais;

    try {

      unencrypted_mac= encrypter.decrypt(this.mac);
      unencrypted_stuff= encrypter.decrypt(this.stuff);
      
      if (!integrity_checker.verify(unencrypted_stuff,unencrypted_mac)) {
	throw(new CryptoException("Message Integrity Violated!"));
      }

      bais= new ByteArrayInputStream(unencrypted_stuff);
      ois= new ObjectInputStream(bais);

      Message m= (Message)ois.readObject();
      
      ois.close();
      bais.close();

      return(m);
    }
    catch(CryptoException ce) {
      ce.printStackTrace();
      throw(ce);
    }
    catch(Exception e) {
      e.printStackTrace();
      throw(new CryptoException("Could not decrypt Message: " + e.toString()));
    }
  }

}
    
