/* Electronic Voting Package
 * The Log Viewer
 * written by Ben Adida (ben@mit.edu)
 */

package evote.network.server;

import evote.network.server.*;
import evote.lang.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

/** 
 * A Log Viewer/Parser
 *
 * This is a simple GUI that reads in a Log file created by <pre>Log</pre>
 * and displays the entries in the log.
 * This would usually be run as a standalone application, after execution
 * of the server that created the log terminated.
 *
 * @see evote.network.server.Log
 */
public class LogViewer extends Frame implements ActionListener, WindowListener, ItemListener {

  /* Log Data variables */
  private Vector log_entries;

  /* GUI variables */
  private List log_list;
  private Button quit_button;

  /** 
   * This creates a LogViewer from an inputstream
   * 
   * @argument is The inputstream from which the LogViewer reads
   *      the log entries
   */
  public LogViewer(InputStream is) throws LogException {
    super("Log Viewer");

    log_entries= new Vector();

    ObjectInputStream ois= null;
    try {
      ois= new ObjectInputStream(is);
      while(true) {
	log_entries.addElement((LogEntry)ois.readObject());
      }
    }
    catch(EOFException eof) {
    }
    catch(Exception ioe) {
      throw(new LogException("Could not create the Log Viewer: "+ ioe.toString()));
    }
    finally {
      try {
	ois.close();
      }
      catch(Throwable t) {
      }
    }

    setupLogList();
    initGUI();
  }

  private void setupLogList() {
    /* We dont use Enumeration here because we WANT
     * to keep the order, and even though Enumeration does 
     * do things in order, that's not in the specs, so we're
     * not going to depend on something that's not spec'ed
     */

    LogEntry le;
    String status;

    log_list= new List();

    for (int i=0; i<log_entries.size(); i++) {
      le= (LogEntry)log_entries.elementAt(i);

      switch(le.getStatus()) {
      case LogEntry.NORMAL_LOG:  status= " normal "; break;
      case LogEntry.WARNING_LOG: status= " warning"; break;
      case LogEntry.SERIOUS_LOG: status= " serious"; break;
      case LogEntry.FATAL_LOG:   status= " fatal  "; break;
      default: status= " normal ";
      }

      log_list.add(le.getDate().toString() + " - " + status + " " + le.getDescription().substring(0,Math.min(20,le.getDescription().length())) + "...");
    }
  }

  private void initGUI() {
    this.setLayout(new BorderLayout());
    log_list.addActionListener(this);
    this.add("Center",log_list);
    
    Panel bottom_p= new Panel();
    quit_button= new Button("Quit");
    quit_button.addActionListener(this);
    bottom_p.add(quit_button);
    
    this.add("South", bottom_p);
    this.pack();
    this.setSize(500,400);
    this.show();
  }

  public void actionPerformed(ActionEvent e) {
    Object source= e.getSource();

    /* Then this is an item */
    if (source == log_list) {
      int which_one= log_list.getSelectedIndex();
      new LogEntryViewer((LogEntry)log_entries.elementAt(which_one));
      return;
    }
    
    if (source == quit_button) {
      this.dispose();
      System.exit(0);
    }
  }

  public static void main(String argv[]) {
    try {
      FileInputStream fis= new FileInputStream(argv[0]);
      new LogViewer(fis);
    }
    catch(LogException e) {
      System.err.println("incorrect file format: " + e.toString());
    }
    catch(Exception e) {
      System.err.println("IO error: "+ e.toString());
    }
  }

  public void itemStateChanged(ItemEvent e) {
  }
     
  public void windowActivated(WindowEvent e) {
  }

  public void windowClosed(WindowEvent e) {
  }
  
  public void windowClosing(WindowEvent e) {
  }

  public void windowDeactivated(WindowEvent e) {
  }

  public void windowDeiconified(WindowEvent e) {
  }

  public void windowIconified(WindowEvent e) {
  }

  public void windowOpened(WindowEvent e) {
  }
}


class LogEntryViewer extends Frame implements WindowListener,ActionListener {
  
  private LogEntry my_entry;

  private Button dismiss_button;

  public LogEntryViewer(LogEntry le) {
    super("Log Entry");
    this.my_entry= le;
    this.addWindowListener(this);
    this.display();
  }

  private void display() {
    Panel top_p= new Panel();
    top_p.setLayout(new GridLayout(2,1));
    
    String status;
    
    switch(my_entry.getStatus()) {
    case LogEntry.NORMAL_LOG:  status= " normal "; break;
    case LogEntry.WARNING_LOG: status= " warning"; break;
    case LogEntry.SERIOUS_LOG: status= " serious"; break;
    case LogEntry.FATAL_LOG:   status= " fatal  "; break;
    default: status= " normal ";
    }

    top_p.add(new Label("Date: " + my_entry.getDate().toString()));
    top_p.add(new Label("Status: " + status));

    this.add("North",top_p);

    TextArea ta= new TextArea(my_entry.getDescription());
    ta.setEditable(false);
    this.add("Center", ta);

    Panel bottom_p= new Panel();
    dismiss_button= new Button("Dismiss");
    dismiss_button.addActionListener(this);
    bottom_p.add(dismiss_button);
    this.add("South", bottom_p);

    this.pack();
    this.setSize(300,300);
    this.show();

  }

  public void actionPerformed(ActionEvent e) {
    this.dispose();
  }

  public void windowActivated(WindowEvent e) {
  }

  public void windowClosed(WindowEvent e) {
  }
  
  public void windowClosing(WindowEvent e) {
    this.dispose();
  }

  public void windowDeactivated(WindowEvent e) {
  }

  public void windowDeiconified(WindowEvent e) {
  }

  public void windowIconified(WindowEvent e) {
  }

  public void windowOpened(WindowEvent e) {
  }
}
