/* Electronic Voting Project
 * Secure Server Code
 * written by Ben Adida (ben@mit.edu)
 */

package evote.network.server;

import java.io.*;
import java.net.*;
import evote.util.queue.*;
import evote.crypto.*;
import evote.util.genrand.*;

/**
 * A Secure Server
 * 
 * The Secure Server is a Server which supports secure connections
 * using SecureConnection. A SecureServer can be made to accept only
 * secure connections, or a mix of secure and insecure connections.
 * <p>
 * Other than that, a SecureServer supports exactly the same exact
 * features as Server, and also needs a ConnectionHandler, which does not
 * need to be any different than the one used for the normal Server.
 *
 * @see SecureConnection
 */
public class SecureServer extends Server {
  
  private RsaKeys server_key;
  private GenRand my_gr= null;
  private boolean accept_insecure= false;

  /**
   * Create a SecureServer, giving all the possible arguments.
   *
   * @argument name The name of the secure server
   * @argument port The port number
   * @argument ch The Connection handler
   * @argument num_threads The number of threads needed in the Server
   * @argument queue_size The size of the request queue
   * @argument rsk The RSA Key Pair for this Server. 
   * This must contain the private part of the key pair.
   * @argument gr The random number generator.
   * @return a new secure server
   * @exception evote.network.server.ServerException
   *   If anything goes wrong in creating the Secure Server,
   *   this exception is thrown.
   */
  public SecureServer(String name, int port, ConnectionHandler ch, int num_threads, int queue_size, RsaKeys rsk, GenRand gr, boolean interface_p) throws ServerException {
    super("SECURE-" + name, port, ch, num_threads, queue_size,interface_p);
    this.server_key= rsk;
    my_gr= gr;
  }    

  public SecureServer(String name, int port, ConnectionHandler ch, int num_threads, int queue_size, RsaKeys rsk, GenRand gr) throws ServerException {
    this(name,port,ch,num_threads,queue_size,rsk,gr,true);
  }

  /**
   * Create a SecureServer, without a random number generator.
   * The Secure Server will create its own random number generator.
   *
   * @argument name The name of the secure server
   * @argument port The port number
   * @argument ch The Connection handler
   * @argument num_threads The number of threads needed in the Server
   * @argument queue_size The size of the request queue
   * @argument rsk The RSA Key Pair for this Server. 
   * This must contain the private part of the key pair.
   * @argument gr The random number generator.
   * @return a new secure server
   * @exception evote.network.server.ServerException
   *   If anything goes wrong in creating the Secure Server,
   *   this exception is thrown.
   */
  public SecureServer(String name, int port, ConnectionHandler ch, int num_threads, int queue_size, RsaKeys rsk, boolean interface_p) throws ServerException {
    this(name, port, ch, num_threads, queue_size, rsk, new GenRand(), interface_p);
    this.my_gr.seed();
  }

  public SecureServer(String name, int port, ConnectionHandler ch, int num_threads, int queue_size, RsaKeys rsk) throws ServerException {
    this(name,port,ch,num_threads,queue_size,rsk, true);
  }

  /**
   * Create a SecureServer, without the RSA Key Pair.
   * The secure server will create its own RSA Key Pair.
   *
   * @argument name The name of the secure server
   * @argument port The port number
   * @argument ch The Connection handler
   * @argument num_threads The number of threads needed in the Server
   * @argument queue_size The size of the request queue
   * @argument rsk The RSA Key Pair for this Server. 
   * This must contain the private part of the key pair.
   * @argument gr The random number generator.
   * @return a new secure server
   * @exception evote.network.server.ServerException
   *   If anything goes wrong in creating the Secure Server,
   *   this exception is thrown.
   */
  public SecureServer(String name, int port, ConnectionHandler ch, int num_threads, int queue_size, GenRand gr, boolean interface_p) throws ServerException {
    this(name, port, ch, num_threads, queue_size, new RsaKeys(gr), gr, interface_p);
  }

  public SecureServer(String name, int port, ConnectionHandler ch, int num_threads, int queue_size, GenRand gr) throws ServerException {
    this(name, port, ch, num_threads, queue_size, gr, true);
  }

  /**
   * Determine whether this server accepts insecure connections
   * By default, a Secure Server refuses insecure connections.
   *
   * @argument b whether or not to accept insecure connections.
   */
  public void setAcceptInsecure(boolean b) {
    accept_insecure= b;
  }

  /** 
   * Make a new Connection for this Server.
   * This creates SecureConnections, with the proper
   * accept_insecure argument, set in the server
   */
  public Connection newConnection(Socket s) throws ConnectionException {
    return(new SecureConnection(s, server_key, my_gr, accept_insecure));
  }

}
