/*
 *  GenRand.java
 *  Last Revised:  20 May 97
 *  Randy L. Milbert <rmilbert@mit.edu>
 */

package evote.util.genrand;

import evote.util.constants.*;
import evote.crypto.*;
import java.awt.*;
import java.applet.Applet;
//import java.security.*;

/**
	The GenRand class generates cryptographically secure random numbers by
	opening a dialog box that tracks the user's cursor movements and hashing
	that to create a seed for the SecureRandom() method in Sun's JDK.  It
	may be instantiated by inserting a line like the following in your source
	code:  "GenRand gr = new GenRand()".
*/
public class GenRand {
	/**
		The MessageDigest object md is used to hash the bytes generated by
		the user moving the cursor using the SHA algorithm.
	*/
  //private MessageDigest md;
	/**
		The SecureRandom object sr is a link to Sun's pseudo random number
		generator (PRNG) and is used whenever the nextBytes(int numBytes)
		method is called.
	*/
	private SecureRandom sr;

	/**
		Call the seed() method to open a dialog that tracks the user's cursor
		movements until a certain number of bytes of data is generated.  Those
		bytes are then hashed using SHA (as provided in Sun's JDK).  Finally,
		that 160-bit hash is used to seed Sun's SecureRandom() pseudo random
		number generator (PRNG).
	*/
	public void seed() {
		/**
			The Frame object f is used as a dummy frame and is necessary to
			create the GetCursorBytes dialog.
		*/
		Frame f = new Frame();
		// First step:  get random bytes from cursor movement.
		GetCursorBytes gcb = new GetCursorBytes(f);
		// display the dialog
		gcb.show();
		
		// second step:  hash the bytes using SHA
// 		try {
// 		  md = MessageDigest.getInstance("SHA");
// 		}
// 		catch(Exception e) {
// 			System.err.println("Error using SHA.");
// 			System.exit(0);
// 		}
		/**
			The byte array hashOut contains the 160-bit hash of the byte array
			generated by the user moving the cursor.
		*/
		
		SHADigest md= new SHADigest();
		byte hashOut[] = md.performHash(gcb.mouseData);

		// third step:  seed the SecureRandom PRNG
		sr = new SecureRandom(hashOut);
		
		f.dispose();
	}

	/**
		Using the SecureRandom() pseudo random number generator supplied by Sun,
		the specified number of random bytes are returned.  The seed() method
		must be called before the nextBytes() function is called.
	*/
	synchronized public byte[] nextBytes(int numBytes) {
		byte randomBytes[] = new byte[numBytes];

		sr.nextBytes(randomBytes);
		return randomBytes;
	}

        public static void main(String S[]) {
                GenRand gr = new GenRand();
		gr.seed();
	        System.out.println("Twenty bytes returned from nextBytes(20):");
	        byte result[] = gr.nextBytes(20);
		
                for (int i = 0; i < 20; i++)
			System.out.println(""+result[i]);
                System.out.println("");
        }
}

/**
	The GetCursorBytes dialog is used by GenRand to track the user's cursor
	movements and store that data in a byte array.  It should not be called
	by your program.  Use GenRand's seed() method instead.
*/
class GetCursorBytes extends Dialog {
	/**
		The numCursorBytes variable is the number of bytes to store using
		the coordinates of the cursor as the user moves it within the dialog.
		The default value is 200.  It must be a multiple of 2.
	*/
	public int numCursorBytes = Constants.genrandNumCursorBytes;
	/**
		Array of bytes to store the coordinates of cursor.
	*/
	public byte mouseData[] = new byte[numCursorBytes];
	/**
		Index for use with mouseData byte array.
	*/
	private int i = 0;
	/**
		The offscreen canvas used in double buffering.
	*/
	private Image OffscreenImage;
	/**
		The interface to the offscreen canvas.
	*/
	private Graphics OffscreenGraphics;
	/**
		The Font object TextFont is used to determine the width and height of
		the text in the dialog.
	*/
        private Font TextFont;
	/**
		The FontMetrics object TextFontMetrics contains metric information about
		the font used in the dialog.
	*/
	private FontMetrics TextFontMetrics;
	/**
		The maximum height of a character can be found by adding the leading space,
		maximum ascent, and maximum descent of the dialog's font.
	*/
	private int maxFontHeight;
	/**
		The width of the window is determined by using information about the metrics
		of the dialog's font.
	*/
	private int winWidth;
	/**
		The height of the window is determined by using information about the metrics
		of the dialog's font.
	*/
	private int winHeight;

	String line1 = new String("To securely send your vote, it is necessary to generate");
	String line2 = new String("random data.  The cursor's position will be continually");
	String line3 = new String("tracked until enough data is obtained.  To ensure");
	String line4 = new String("randomness, move the cursor quickly and erratically");
	String line5 = new String("within the boundary of this window.");
	
	DepressedBox border = new DepressedBox();

	ProgressBar bar = new ProgressBar();

	/**
		The constructor takes a frame as an argument because a frame is
		required to create a dialog.  The constructor sets the font and
		size of the dialog and the size and position of the progress bar.
	*/
	public GetCursorBytes(Frame parent) {
		// set the title and set as modal (force user input)
		super(parent, "Random Number Seed Generation", true);
		setLayout(new BorderLayout());

                //String font_name[] = TextFontMetrics.getFontList();
                TextFont = new Font("Dialog", Font.PLAIN, 12);
                TextFontMetrics = this.getFontMetrics(TextFont);
		maxFontHeight = TextFontMetrics.getLeading() + TextFontMetrics.getMaxAscent() + TextFontMetrics.getMaxDescent();
		setFont(TextFont);
		
		winWidth = TextFontMetrics.stringWidth(line1) + 50;

		// initialize the progress bar and border
		border.width	= bar.width		= winWidth - 20;
		border.height	= bar.height	= 15;
		border.x		= bar.x			= 10;
		border.y		= bar.y			= 60 + (maxFontHeight) * 5 + 10;

		winHeight = 60 + (maxFontHeight) * 5 + 10 + bar.height + 60;

		resize(winWidth, winHeight);
	}

	/**
		Whenever the mouse is moved, the x and y coordinates are cast to
		bytes and these bytes are stored in an array.  The screen is repainted
		so that the progress bar can be updated.  When enough bytes have been
		generated, the dialog is hidden with the hide() method.
	*/
	public boolean mouseMove(Event evt, int x, int y) {
		if (i < numCursorBytes) {
			// store the least significant bytes of the x and y cursor coordinates
			mouseData[i++] = (byte) x;
			mouseData[i++] = (byte) y;
			repaint();
		}
		else
			hide();

		return true;
	}

	/**
		The update method is used in double buffering to prevent text and progress
		bar flicker.  An image with dimensions equal to that of the dialog is
		created and all drawing is done to that image.  When there is a break
		in the action, that image is swapped to the screen.
	*/
	public synchronized void update(Graphics g) {
		if (OffscreenImage == null) {
			// make a new image the same size as the dialog
			OffscreenImage = createImage(winWidth,winHeight);
			// bind our interface to the Image
			OffscreenGraphics = OffscreenImage.getGraphics();
		}

		// do applet painting to our canvas
		paint(OffscreenGraphics);
		// paint the real graphics window
		g.drawImage(OffscreenImage,0,0,this);
	}

	/**
		When the screen is repainted, the text is drawn to the screen and
		the progress bar is updated.
	*/
	public void paint(Graphics g) {
		// paint over the old image
		g.setColor(Color.lightGray);
		g.fillRect(0,0,winWidth-1,winHeight-1);

		g.setColor(Color.black);

		g.drawString(line1,10,55);
		g.drawString(line2,10,70);
		g.drawString(line3,10,85);
		g.drawString(line4,10,100);
		g.drawString(line5,10,115);

		bar.percent = (float) i / (float) numCursorBytes;
		border.drawBorder(g);
		bar.drawBar(g);
	}
}

/**
	The DepressedBox class is used to draw a 3D border around the progress
	bar.  The width, height, x, and y variables must all be set before using
	drawBorder to actually draw the border.  What is drawn is a 2 pixel wide
	border surrounding the specified rectangle.
*/
class DepressedBox {
	/**
		The width of the depressed box border.
	*/
	public static final int borderWidth = 2;
	/**
		The x and y coordinates of the upper left corner of the rectangle to
		border.
	*/
	public int x, y;
	/**
		The width and height of the rectangle to draw a border around.
	*/
	public int width, height;
	/**
		The dark shading color.  The default is dark gray.
	*/
	public Color DarkShading = new Color(128,128,128);

	/**
		After instantiating DepressedBox and setting the width, height, x, and y
		variables, drawBorder(Graphics g) may be called to actually draw the
		3D rectangular border.
	*/
	public void drawBorder(Graphics g) {
		int minX = x - borderWidth;
		int minY = y - borderWidth;
		int maxX = x + width + borderWidth;
		int maxY = y + height + borderWidth;

		g.setColor(DarkShading);
		g.drawLine(minX, minY, maxX-1, minY);
		g.drawLine(minX, minY, minX, maxY-1);
		
		g.setColor(Color.black);
		g.drawLine(minX+1, minY+1, maxX-borderWidth, minY+1);
		g.drawLine(minX+1, minY+1, minX+1, maxY-borderWidth);
		
		g.setColor(Color.lightGray);
		g.drawLine(minX, maxY-1, maxX, maxY-1);
		g.drawLine(maxX, minY, maxX, maxY-1);
		g.drawLine(minX+1, maxY-borderWidth, maxX-borderWidth+1, maxY-borderWidth);
		g.drawLine(maxX-1, minY+1, maxX-1, maxY-borderWidth-1);
	}
}

/**
	The ProgressBar class draws the progress bar.  After instantiating the
	object and setting the width, height, x, and y coordinates, call drawBar
	to actually do the drawing.  The percent variable is a floating point
	number in the range [0,1] and represents the percent of the progress
	bar that should be drawn.
*/
class ProgressBar {
	/**
		The x and y coordinates of the upper left corner of the progress bar.
	*/
	public int x, y;
	/**
		The width and height of the progress bar when complete.
	*/
	public int width, height;
	/**
		The color of the progress bar.  The default is blue.
	*/
	public Color BarColor = new Color(0,0,255);
	/**
		The percent of progress that should be drawn.  It should be a floating
		point number in the range of [0,1].
	*/
	public float percent;

	/**
		Draws the rectangle defined by the width, height, x, and y coordinates
		and using the color set by BarColor (the default is blue).
	*/
	public void drawBar(Graphics g) {
		g.setColor(BarColor);
		g.fillRect(x, y, (int) (width * percent), height);
	}
}
